/*
	File:		CappuccinoAction.h

	Contains:	Classes for undo/redo support.

	Written by:	Troy Gaul
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/

#ifndef _CAPPUCCINOACTION_
#define _CAPPUCCINOACTION_

//------------------------------------------------------------------------------
// Forward declarations
//------------------------------------------------------------------------------

class CAction;
class Cappuccino;
class CCappuccinoContent;
class CSettings;

class Environment;
class ODSession;

//------------------------------------------------------------------------------
// ActionCode
//------------------------------------------------------------------------------

typedef ODUShort ActionCode;

//------------------------------------------------------------------------------
// CAction
//------------------------------------------------------------------------------

class CAction {

public:

	virtual ~CAction();
	
	// -- Performing --
	
	void Perform(Environment* ev) { this->HandleDo(ev); }
	
	virtual void HandleDo(Environment* ev);
	
	virtual void HandleUndo(Environment* ev);
	
	virtual void HandleRedo(Environment* ev);
	
	// -- Utility --
	
	ODActionData* CreateActionData();

	static CAction* GetActionStateAction(ODActionData* actionState);

protected:
	
	CAction(Cappuccino* part, ActionCode);

	// -- Performing (override these for subclasses) --
	
	virtual void Do(Environment* ev) = 0;
	
	virtual void Undo(Environment* ev) = 0;
	
	virtual void Redo(Environment* ev) = 0;

	// -- Misc. --

	virtual ODName* GetUndoRedoName(Environment* ev, ODBoolean isUndo);
	
	virtual void AddToActionHistory(Environment* ev, 
									ODActionType actionType = kODSingleAction);
	
protected:

	Cappuccino*		fPart;
	ODSession*		fSession;
	ActionCode		fActionCode;
};

//------------------------------------------------------------------------------
// CTextChangeAction
//------------------------------------------------------------------------------

class CTextChangeAction : public CAction {

	typedef CAction Inherited;

public:

	CTextChangeAction(Cappuccino* part, ActionCode,
					  CCappuccinoContent* newContent);
	
	virtual ~CTextChangeAction();
	
	// -- Performing --

	virtual void Do(Environment* ev);
	
	virtual void Undo(Environment* ev);
	
	virtual void Redo(Environment* ev);
	
protected:

	CCappuccinoContent*		fPreviousContent;
	CCappuccinoContent*		fCurrentContent;
};

//------------------------------------------------------------------------------
// CClipboardTextChangeAction
//------------------------------------------------------------------------------

class CClipboardTextChangeAction : public CTextChangeAction {

	typedef CTextChangeAction Inherited;

public:

	CClipboardTextChangeAction(Cappuccino* part, ActionCode, ODCloneKind, 
							   CCappuccinoContent*);
	
	virtual ~CClipboardTextChangeAction();
	
	// -- Performing --
	
	virtual void HandleDo(Environment* ev);
	
	virtual void HandleUndo(Environment* ev);
	
	virtual void HandleRedo(Environment* ev);

private:

	ODUpdateID		fUpdateID;
	ODCloneKind		fCloneKind;

};

//------------------------------------------------------------------------------
// CCutAction
//------------------------------------------------------------------------------

class CCutAction : public CClipboardTextChangeAction {

	typedef CClipboardTextChangeAction Inherited;

public:

	CCutAction(Cappuccino* part);

};

//------------------------------------------------------------------------------
// CPasteAction
//------------------------------------------------------------------------------

class CPasteAction : public CClipboardTextChangeAction {

	typedef CClipboardTextChangeAction Inherited;

public:

	CPasteAction(Cappuccino* part, CCappuccinoContent*);

};

//------------------------------------------------------------------------------
// CClearAction
//------------------------------------------------------------------------------

class CClearAction : public CTextChangeAction {

	typedef CTextChangeAction Inherited;

public:

	CClearAction(Cappuccino* part);

};

//------------------------------------------------------------------------------
// CDropAction
//------------------------------------------------------------------------------

class CDropAction : public CTextChangeAction {

	typedef CTextChangeAction Inherited;

public:

	CDropAction(Cappuccino* part, CCappuccinoContent*);

};

//------------------------------------------------------------------------------
// CDragBeginAction
//------------------------------------------------------------------------------

class CDragBeginAction : public CAction {

	typedef CAction Inherited;

public:

	CDragBeginAction(Cappuccino* part);
	
	// -- Performing --
	
	virtual void HandleDo(Environment* ev);

protected:

	virtual void Do(Environment* ev) { }

	virtual void Undo(Environment* ev) { }

	virtual void Redo(Environment* ev) { }
};

//------------------------------------------------------------------------------
// CDragEndAction
//------------------------------------------------------------------------------

class CDragEndAction : public CAction {

	typedef CAction Inherited;

public:

	CDragEndAction(Cappuccino* part);
	
	// -- Performing --
	
	virtual void HandleDo(Environment* ev);
	
protected:

	virtual void Do(Environment* ev) { }

	virtual void Undo(Environment* ev) { }

	virtual void Redo(Environment* ev) { }
};

//------------------------------------------------------------------------------
// CSetTextAction
//------------------------------------------------------------------------------

class CSetTextAction : public CTextChangeAction {

	typedef CTextChangeAction Inherited;

public:

	CSetTextAction(Cappuccino* part, CCappuccinoContent*);

};

//------------------------------------------------------------------------------
// CSettingsChangeAction
//------------------------------------------------------------------------------

class CSettingsChangeAction : public CAction {

	typedef CAction Inherited;

public:

	CSettingsChangeAction(Cappuccino* part, CSettings* newSettings);
	
	virtual ~CSettingsChangeAction();
	
	// -- Performing --

	virtual void Do(Environment* ev);
	
	virtual void Undo(Environment* ev);
	
	virtual void Redo(Environment* ev);
	
protected:

	CSettings*		fPreviousSettings;
	CSettings*		fCurrentSettings;
};


#endif