/*
	File:		CappuccinoUtils.cpp

	Contains:	Cappuccino utility functions & classes

	Written by:	Steve Smith and Troy Gaul

	Copyright:	 1994 - 1995 by Apple Computer, Inc., all rights reserved.
*/

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (ie. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with the "ev" parameter set.
#include <Except.h>
#endif

// --- Cappuccino Includes ---

#ifndef _CAPPUCCINOUTILS_
#include "CappuccinoUtils.h"
#endif

#ifndef _CAPPUCCINODEF_
#include "CappuccinoDef.h"
#endif

// --- OpenDoc Includes ---

#ifndef SOM_ODArbitrator_xh
#include <Arbitrat.xh>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef SOM_ODDocument_xh
#include <Document.xh>
#endif

#ifndef SOM_ODDraft_xh
#include <Draft.xh>
#endif

#ifndef SOM_ODNameSpaceManager_xh
#include <NmSpcMg.xh>
#endif

#ifndef SOM_ODContainer_xh
#include <ODCtr.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdTypes_defined
#include <StdTypes.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include <StdDefs.xh>
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

#ifndef SOM_ODStorageUnitView_xh
#include <SUView.xh>
#endif

#ifndef SOM_ODValueNameSpace_xh
#include <ValueNS.xh>
#endif

#ifndef SOM_ODWindowState_xh
#include <WinStat.xh>
#endif

// -- OpenDoc Utilities --

#ifndef _BNDNSUTL_
#include <BndNSUtl.h>
#endif

#ifndef _DLOGUTIL_
#include <DlogUtil.h>
#endif

#ifndef _DOCUTILS_
#include <DocUtils.h>
#endif

#ifndef _ISOSTR_
#include <ISOStr.h>
#endif

#ifndef _ODDEBUG_
#include <ODDebug.h>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef _STDTYPIO_
#include <StdTypIO.h>
#endif

#ifndef _STORUTIL_	
#include <StorUtil.h>
#endif

#ifndef _TEMPOBJ_
#include <TempObj.h>
#endif

#ifndef _TEMPITER_
#include <TempIter.h>
#endif

#ifndef _USERSRCM_
#include <UseRsrcM.h>
#endif

// --- Macintosh Includes ---

#ifndef __GESTALT__
#include <Gestalt.h>
#endif

#ifndef __GXMATH__
#include <GXMath.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

#ifndef __SPEECH__
#include <Speech.h>
#endif

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif


#pragma segment CappuccinoUtilities


//==============================================================================
// Utility Functions
//==============================================================================
#pragma mark  Utility Functions 

//------------------------------------------------------------------------------
// GetPartName
//------------------------------------------------------------------------------

ODIText* GetPartName(Environment* ev, ODPart* part, ODType category)
{
	ASSERT(part != kODNULL, kODErrIllegalNullPartInput);

	// Get the root part of the document.
	ODDraft* draft = ODGetDraft(ev,part);
	TempODPart rootPart = ODAcquireRootPartOfDraft(ev, draft);
	
	// Get the part name.
	// If we can't get a valid name for the part, we generate one using
	// the user visible category name of the part's category.
	TempODIText partName = ODGetITextProp(ev, part->GetStorageUnit(ev), 
										 kODPropName, kODMacIText, kODNULL);
	
	// If the part is the root of the document, return the name of the file.
	if ( ODObjectsAreEqual(ev, part, rootPart) )
	{
		ODContainer* container = draft->GetDocument(ev)->GetContainer(ev);
		TempPlatformFile file = GetPlatformFileFromContainer(ev, container);

		TempODIText fileName = file->GetName();
		
		// Test the file name against the part name. If the two are equivalent
		// (not equal), then use the part name.
		if ( !NamesAreEquivalent(ev, fileName, partName) )
		{
			// If the names are different, return the file name.
			// (This code transfers the ODIText object to the partName tempobj)
			DisposeIText(partName.DontDelete());
			partName = fileName.DontDelete();
		}
	}
	else
	{
		if ( (partName == kODNULL) || (GetITextStringLength(partName) == 0) )
		{
			ODIText* categoryName;
			ODNameSpaceManager* nsMgr = ODGetSession(ev,part)->GetNameSpaceManager(ev);
			
			// Get the category string from the category name space.
			if ( GetUserCatFromCat(nsMgr, category, &categoryName) )
			{
				// If we successfully retrieved the category user string, return it.
				// (This code transfers the ODIText object to the partName tempobj)
				DisposeIText(partName.DontDelete());
				partName = categoryName;
			}
			else
			{
	#if ODDebug
				// This should never happen. Check NMAP for errors.
				DebugStr("\pCategory NMAP bad, or Preferences corrupted.");
	#else
				THROW(kODErrInvalidNSType);
	#endif
			}
		}
	}
	
	return partName.DontDelete();
}

//------------------------------------------------------------------------------
// NamesAreEquivalent
//------------------------------------------------------------------------------

ODBoolean NamesAreEquivalent(Environment* ev, ODIText* fileName,
										ODIText* partName)
{
	Str255 fileStr;
	Str255 partStr;
	
	// If the strings are in different languages, we're done.
	if ( (GetITextScriptCode(fileName) != GetITextScriptCode(partName)) ||
			GetITextLangCode(fileName) != GetITextLangCode(partName) )
		return kODFalse;
	
	GetITextPString(fileName, fileStr);
	GetITextPString(partName, partStr);

	// If the string lengths are different, we're done.
	if ( fileStr[0] != partStr[0] )
		return kODFalse;

	// Return the Toolbox string equivalence test.
	return EqualString(fileStr,partStr,kODTrue,kODTrue);
}

//------------------------------------------------------------------------------
// GetEditorScriptLanguage
//------------------------------------------------------------------------------

void GetEditorScriptLanguage(Environment* ev, ODScriptCode* script,
										ODLangCode* language)
{
	long region;
	
	ODSLong rfRef;
	rfRef = BeginUsingLibraryResources();
	{
		Handle versHdl = Get1Resource('vers', 1);
		
		// Get the region code of the editor, otherwise use the
		// region code the of the primary system script.
		
		if ( versHdl )
		{
			region = (long)(*(VersRecHndl)versHdl)->countryCode;
			ReleaseResource(versHdl);
		}
		else
		{
			region = GetScriptManagerVariable(smRegionCode);
		}
		
		// Spanish & Japanese are not actually supported by the editor.
		// They are provided as examples of how to add recognition of
		// additional regions (see Script.h for region codes).
		
		switch ((short)region) {
			case verUS:
				*script = smRoman;
				*language = langEnglish;
				break;
			case verSpain:
				*script = smRoman;
				*language = langSpanish;
				break;
			case verJapan:
				*script = smJapanese;
				*language = langJapanese;
				break;
			default:
				*script = smRoman;
				*language = langEnglish;
		}				
	}
	EndUsingLibraryResources(rfRef);
}

//------------------------------------------------------------------------------
// FixedToIntRect
//------------------------------------------------------------------------------

void FixedToIntRect(ODRect& fixedRect, Rect& intRect)
{
	intRect.top		= FixedToInt(fixedRect.top);
	intRect.left	= FixedToInt(fixedRect.left);
	intRect.bottom	= FixedToInt(fixedRect.bottom);
	intRect.right	= FixedToInt(fixedRect.right);
}

//------------------------------------------------------------------------------
// IntToFixedRect
//------------------------------------------------------------------------------

void IntToFixedRect(Rect& intRect, ODRect& fixedRect)
{
	fixedRect.left		= ff(intRect.left);
	fixedRect.top		= ff(intRect.top);
	fixedRect.right		= ff(intRect.right);
	fixedRect.bottom	= ff(intRect.bottom);
}

//------------------------------------------------------------------------------
// LoadThumbnail
//------------------------------------------------------------------------------

void LoadThumbnail(Environment* ev, Handle* thumbnail)
{
	if ( *thumbnail ) return;
	
	ODSLong rfRef;
	rfRef = BeginUsingLibraryResources();
	{
		*thumbnail = (Handle) GetPicture(kThumbnailPicture);
	}
	EndUsingLibraryResources(rfRef);
}

//------------------------------------------------------------------------------
// TilePartWindow
//------------------------------------------------------------------------------

Rect TilePartWindow(Environment* ev, Rect* facetBounds, Rect* partWindowBounds)
{
	const short	kWindowTilingConst	= 20;
	const short kLeftToRight		= 0;
	const short kRightToLeft		= -1;
	
	short direction;
	
	// Get the direction for the primary script system running on this machine.
	// (Right-to-Left or Left-to-Right)
	direction = GetSysDirection();
	
	// The child window should be tiled from the topLeft corner of the 
	// active facet whose frame is being opened.
	if ( direction == kLeftToRight )
	{
		// Position the window rect at the top/left corner of the facet.
		OffsetRect(partWindowBounds, facetBounds->left, facetBounds->top);
		// Now tile the window rect down and to the right.
		OffsetRect(partWindowBounds, kWindowTilingConst, kWindowTilingConst);
	}
	// The child window should be tiled from the topRight corner of the 
	// active facet whose frame is being opened.
	else if ( direction == kRightToLeft )
	{
		// Position the window rect at the top/right corner of the facet.
		OffsetRect(partWindowBounds, (partWindowBounds->right - facetBounds->right),
					facetBounds->top);
		// Now tile the window rect down and to the left.
		OffsetRect(partWindowBounds, -kWindowTilingConst, kWindowTilingConst);
	}
	
	return *partWindowBounds;
}

//------------------------------------------------------------------------------
// CountFacets
//------------------------------------------------------------------------------

ODUShort CountFacets(Environment* ev, ODFrame* frame)
{
	ODUShort facetCount = 0;
	ODFacet* facet;
	
	TempODFrameFacetIterator ffiter(ev, frame);
	facet = ffiter.First();
	while ( ffiter.IsNotComplete() )
	{
		facetCount++;
		facet = ffiter.Next();
	}
	
	return facetCount;
}

//------------------------------------------------------------------------------
// HasFocusFromSet
//------------------------------------------------------------------------------

ODBoolean HasAnyFocusInSet(Environment* ev, ODFrame* frame, ODFocusSet* set)
{
	ASSERT_NOT_NULL(frame);
	ASSERT_NOT_NULL(set);

	ODBoolean hasFocus = kODFalse;
	
	TempODFocusSetIterator iter(ev, set);
	for ( ODTypeToken focus = iter.First(); iter.IsNotComplete(); 
		  focus = iter.Next() )
	{
		TempODPart part = frame->AcquirePart(ev);
		ODArbitrator* arbitrator = ODGetSession(ev, part)->GetArbitrator(ev);
	
		TempODFrame focusFrame = arbitrator->AcquireFocusOwner(ev, focus);
		if ( ODObjectsAreEqual(ev, focusFrame, frame) )
		{
			hasFocus = kODTrue;
			break;
		}
	}
	
	return hasFocus;
}

//------------------------------------------------------------------------------
// SetPreferredKind
//------------------------------------------------------------------------------

void SetPreferredKind( Environment*		ev, 
					   ODStorageUnit*	su,
					   ODValueType		kind )
{
	TRY
		if ( kind == kODNULL )
			ODSURemoveProperty(ev, su, kODPropPreferredKind);
		else
			ODSetISOStrProp(ev, su, kODPropPreferredKind, kODISOStr, kind);
	CATCH_ALL
		// Remove the property and value if something went wrong.
		ODSURemoveProperty(ev, su, kODPropPreferredKind);
	ENDTRY
}

//------------------------------------------------------------------------------
// DuplicateISOStr
//------------------------------------------------------------------------------

ODISOStr DuplicateISOStr( ODISOStr string )
{
	// Since ISO strings are just C strings, we call the following ODUtils
	// routine:
	return ODISOStrFromCStr(string);
}

//------------------------------------------------------------------------------
// HasSpeech
//------------------------------------------------------------------------------

ODBoolean HasSpeech()
{
	ODBoolean hasSpeech = kODFalse;

#ifdef powerc
	// There is not yet a CFM-68K shared library for the speech manager, so
	// we must disable that menu item.
	
	long response;
	OSErr error = Gestalt(gestaltSpeechAttr, &response);
	
	if ( error == noErr 
		&& (response & (1 << gestaltSpeechMgrPresent))
		&& (void*) SpeakString != (void*) kUnresolvedCFragSymbolAddress )
	{
		hasSpeech = kODTrue;
	}
#endif
	
	return hasSpeech;
}

//------------------------------------------------------------------------------
// FINF struct
//------------------------------------------------------------------------------

struct FINF 
{
	short	font;
	short	face;
	short	size;
};

//------------------------------------------------------------------------------
// GetIndFINF
// (must have focused resource fork before calling)
//------------------------------------------------------------------------------

static FINF GetIndFINF(short id, short num)
{
	FINF finf;

	finf.font = systemFont;
	finf.face = 0;
	finf.size = 0;

	if (num > 0)
	{
		Handle fh = GetResource('finf', id);
		if ( fh && num <= *(short *)(*fh) )
			finf = ((FINF*) ((*fh) + 2)) [num - 1];
	}
	
	return finf;
}

//------------------------------------------------------------------------------
// SetDialogDefaults
//------------------------------------------------------------------------------

void SetDialogDefaults(DialogPtr dialog, ODUShort finfID, ODUShort index)
{
	FINF finf;

	SetDialogDefaultItem(dialog, ok);
	SetDialogCancelItem(dialog, cancel);
		
	if ( finfID != 0 )
	{	
		SetPort(dialog);
		
		finf = GetIndFINF(finfID, index);
		
		TextFont(finf.font);
		TextSize(finf.size);
		TextFace(finf.face);
		
		DialogPeek dialogPeek = (DialogPeek) dialog;
		(**(dialogPeek->textH)).txFont = finf.font;
		(**(dialogPeek->textH)).txSize = finf.size;
		(**(dialogPeek->textH)).txFace = finf.face;
		
		FontInfo finfo;
		GetFontInfo(&finfo);
		(**(dialogPeek->textH)).lineHeight = finfo.leading + finfo.ascent + finfo.descent;
		(**(dialogPeek->textH)).fontAscent = finfo.ascent;
		
		TECalText(dialogPeek->textH);
	}
}

//------------------------------------------------------------------------------
// IsNumericCharacter
//------------------------------------------------------------------------------

static ODBoolean IsNumericCharacter( UInt32 charKeyCode )
{
	char charCode = charKeyCode & charCodeMask;
	return (charCode >= '0' && charCode <= '9');
}

//------------------------------------------------------------------------------
// IsControlCharacter
//------------------------------------------------------------------------------

static ODBoolean IsControlCharacter( UInt32 charKeyCode )
{
	const char kMaxControlChar = 0x1f;

	char charCode = charKeyCode & charCodeMask;
	return (charCode <= kMaxControlChar);
}

//------------------------------------------------------------------------------
// IntegerDialogFilter
//------------------------------------------------------------------------------

pascal ODBoolean IntegerDialogFilter( DialogPtr			dialog, 
									  EventRecord*		event,
									  short*			itemHit )
{
	ODBoolean result = kODFalse;

	if ( event->what == keyDown )	// just looking for keystrokes
	{
		if ( !IsNumericCharacter(event->message) && !IsControlCharacter(event->message) )
		{
			char charCode = event->message & charCodeMask;
			ODBoolean disallow = kODFalse;
			
			switch ( charCode ) 
			{
				case '.':
					// We need to allow command-., but disallow just '.':
					if ( (event->modifiers & cmdKey) != cmdKey ) 
						disallow = kODTrue;
					break;
		
				default:
					// For all non-numeric keystrokes, beep at the user
					// and clear the event so the char is not typed.
					disallow = kODTrue;
					break;
			}
			
			if ( disallow )
			{
				// In order to disallow the entry, beep at the user
				// and clear the event so the char is not typed.
				SysBeep(1);
				event->what = nullEvent;
			}
		}
	}
	
	if ( result == kODFalse )
		result = ODDialogFilterProc(dialog, event, itemHit);
	
	return result;
}

//==============================================================================
// CTempDialogState helper class
//
//	This is a stack-based object that sets up state for a modal dialog in its
//	constructor, and resets the state in the destructor. It deactivates the 
//	front window, removes the "Redo" item from the Edit menu, and updates the 
//	Mac scrap.
//
//	This is similar to the CWithDialogState class from the printing utility, 
//	but this version also ensapsulates the usage of library resources so that 
//	they will be available for the life of the dialog.  
//==============================================================================
#pragma mark  CTempDialogState 

//------------------------------------------------------------------------------
// CTempDialogState Constructor
//------------------------------------------------------------------------------

CTempDialogState::CTempDialogState( Environment *ev, ODSession *session )
{
	fEv			= ev;
	fSession	= session;
	fDialogPtr	= kODNULL;
	fResRefNum	= -1;
	
	fResRefNum = BeginUsingLibraryResources();
	TempODMenuBar menuBar = fSession->GetWindowState(ev)->AcquireCurrentMenuBar(ev);
	fSession->GetWindowState(ev)->DeactivateFrontWindows(ev);
	ODDialogBegin(ev, session, menuBar, kODNULL);
}

//------------------------------------------------------------------------------
// CTempDialogState Destructor
//------------------------------------------------------------------------------

CTempDialogState::~CTempDialogState( )
{
	if (fDialogPtr)
	{
		DisposeDialog(fDialogPtr);
		fDialogPtr = kODNULL;
	}
	
	ODDialogEnd();
	fSession->GetWindowState(fEv)->ActivateFrontWindows(fEv);
	
	ODSLong ref = fResRefNum;
	if ( ref != -1 ) 			// Make sure it's done only once
	{
		fResRefNum = -1;
		EndUsingLibraryResources(ref);
	}
}

//------------------------------------------------------------------------------
// CTempDialogState::CreateDialog
//------------------------------------------------------------------------------
DialogPtr CTempDialogState::CreateDialog( ODSShort resID )
{
	WASSERT(fDialogPtr == kODNULL);
	
	fDialogPtr = ODGetNewDialog(fEv, resID, fSession);
	THROW_IF_NULL(fDialogPtr);
	
	return fDialogPtr;
}
										  
//==============================================================================
// CFrameProxy
//==============================================================================
#pragma mark  CFrameProxy 

//------------------------------------------------------------------------------
// CFrameProxy::~CFrameProxy
//------------------------------------------------------------------------------

CFrameProxy::~CFrameProxy()
{
	ODSafeReleaseObject(fFrame);
}

//------------------------------------------------------------------------------
// CFrameProxy::InitFrameProxy (ODDraft)
//------------------------------------------------------------------------------

void CFrameProxy::InitFrameProxy(ODID frameID, ODDraft* draft)
{
	ASSERT(draft!=kODNULL, kODErrIllegalNullInput);
	
	fFrame = kODNULL;
	fID = frameID;
	fDraft = draft;
}
	
//------------------------------------------------------------------------------
// CFrameProxy::InitFrameProxy (ODFrame)
//------------------------------------------------------------------------------

void CFrameProxy::InitFrameProxy(Environment* ev, ODFrame* frame)
{
	ASSERT(frame!=kODNULL, kODErrIllegalNullFrameInput);
	
	this->SetFrame(ev,frame);
}

//------------------------------------------------------------------------------
// CFrameProxy::SetFrame
//
// The code will not affect the proxy fields unless it can
// successfully acquire the incoming frame, its id, and
// the draft the frame lives in. If something goes wrong,
// the proxy remains unchanged.
//------------------------------------------------------------------------------

void CFrameProxy::SetFrame(Environment* ev, ODFrame* frame)
{
	ASSERT_NOT_NULL(frame);
	
	// Perform all the operations that can fail first.
	TempODPart	tPart	= frame->AcquirePart(ev);
	ODDraft*	draft	= ODGetDraft(ev, tPart);
	ODID		id		= frame->GetID(ev);
	
	// Release the previously referenced frame and acquire the new one.
	ODTransferReference(ev, fFrame, frame);

	// If all went well, set the proxy fields.
	fFrame = frame;
	fID = id;
	fDraft = draft;
}

//------------------------------------------------------------------------------
// CFrameProxy::GetFrame
//------------------------------------------------------------------------------

ODFrame* CFrameProxy::GetFrame(Environment* ev)
{
	if ( fFrame == kODNULL )
	{
		TRY
			TempODFrame frame = fDraft->AcquireFrame(ev, fID);
			this->SetFrame(ev,frame);
		CATCH_ALL
			fFrame = kODNULL;
			fID = kODNULLID;
		ENDTRY
	}
	return fFrame;
}

//------------------------------------------------------------------------------
// CFrameProxy::FrameIsLoaded
//------------------------------------------------------------------------------

ODBoolean CFrameProxy::FrameIsLoaded(Environment* ev)
{
	ODBoolean loaded = (fFrame != kODNULL ||
						fDraft->IsValidID(ev, fID));
	return loaded;
}

//------------------------------------------------------------------------------
// CFrameProxy::Purge
//------------------------------------------------------------------------------

void CFrameProxy::Purge(Environment* ev)
{
	if ( fFrame != kODNULL )
	{
		fID = fFrame->GetID(ev);
		ODReleaseObject(ev, fFrame);
	}
}

//==============================================================================
// CFrameInfo
//==============================================================================
#pragma mark  CFrameInfo 

//------------------------------------------------------------------------------
// CFrameInfo::CFrameInfo
//------------------------------------------------------------------------------

CFrameInfo::CFrameInfo(ODSession* session)
{
	fSession = session;
	fFrameActive = kODFalse;
	fFrameReactivate  = kODFalse;
	fShouldDisposeWindow = kODFalse;
	fActiveFacet = kODNULL;
	fSourceFrame = kODNULL;
	fDependentFrame = kODNULL;
	fPartWindowID = kODNULLID; 
}

//------------------------------------------------------------------------------
// CFrameInfo::~CFrameInfo
//------------------------------------------------------------------------------

CFrameInfo::~CFrameInfo()
{
	// Deleting the proxies will release the encapsulated frames.
	ODDeleteObject(fDependentFrame);
	ODDeleteObject(fSourceFrame);
}

//------------------------------------------------------------------------------
// CFrameInfo::Externalize
//------------------------------------------------------------------------------

void CFrameInfo::Externalize(Environment* ev, ODStorageUnitView* storageUnitView)
{
	// This method assumes that OpenDoc has passed us a storageUnitView
	// that is focused to a property, but no particular value.
	
	ODStorageUnit* storageUnit = storageUnitView->GetStorageUnit(ev);

	this->CleanseFrameInfoProperty(ev, storageUnit);
	this->ExternalizeFrameInfo(ev, storageUnit, kODNULLKey, kODNULL);
}

//------------------------------------------------------------------------------
// CFrameInfo::CleanseFrameInfoProperty
//------------------------------------------------------------------------------

void CFrameInfo::CleanseFrameInfoProperty( Environment*		ev, 
										   ODStorageUnit*	storageUnit )
{
	ODULong numValues;
	ODULong index;
		
	numValues = storageUnit->CountValues(ev);
	
	for (index = numValues; index >= 1; index--)
	{
		// Index, from 1 to n, through the values.
		storageUnit->Focus(ev, kODNULL, kODPosSame, kODNULL, index, 
						   kODPosUndefined);
								
		// Get the ISO type name for the value. The temp object
		// will automatically delete the returned value when this
		// scope is exited.
		TempODValueType value = storageUnit->GetType(ev);
		
		// If the value type is not one we support, remove it.
		if ( !ODISOStrEqual(value, kCappuccinoInfo) )
			storageUnit->Remove(ev);
	}
}

//------------------------------------------------------------------------------
// CFrameInfo::ExternalizeFrameInfo
//------------------------------------------------------------------------------

void CFrameInfo::ExternalizeFrameInfo( Environment*		ev, 
									   ODStorageUnit*	storageUnit,
									   ODDraftKey		key, 
									   ODFrame*			scopeFrame )
{
	// This method behaves much the same way as the Cappuccino::ExternalizeStateInfo
	// method.
	
	if ( ODSUExistsThenFocus(ev, storageUnit, kODNULL, kCappuccinoInfo) )
	{
		// Persistent object references are stored in a side table, rather than
		// in the property/value stream. Thus, deleting the contents of a value
		// will not "delete" the references previously written to that value. To
		// completely "delete" all references written to the value, we must
		// remove the value and add it back.
		storageUnit->Remove(ev);
	}

	// Add a value to write the data into.
	storageUnit->AddValue(ev, kCappuccinoInfo);
	
	// Write a weak reference to our source frame.
	if ( fSourceFrame )
	{
		ODID frameID = fSourceFrame->GetID();

		// If a draft key exists, then we are being cloned to another draft.
		// We must "weak" clone our display frame and reference the cloned
		// frame. The part re-uses the frameID variable so there aren't two
		// different GetWeakStorageUnitRef calls.
		if ( key )
		{
			ODID	 scopeFrameID = (scopeFrame ? scopeFrame->GetID(ev) : kODNULLID);
			ODDraft* fromDraft = fSourceFrame->GetDraft();
			frameID = fromDraft->WeakClone(ev, key, frameID, kODNULLID, scopeFrameID);
		}
		
		// Write out weak references to each of the part's display frames.
		ODSetWeakSURefProp(ev, storageUnit, kODNULL, kODNULL, frameID);
	}
	else
	{
		ODStorageUnitRef emptyRef = {0,0,0,0};
		StorageUnitSetValue(storageUnit, ev, kODStorageUnitRefSize, &emptyRef);
	}
	
	// Write a weak reference to our dependent frame.
	if ( fDependentFrame )
	{
		ODID frameID = fDependentFrame->GetID();
		// If a draft key exists, then we are being cloned to another draft.
		// We must "weak" clone our display frame and reference the cloned
		// frame. The part re-uses the frameID variable so there aren't two
		// different GetWeakStorageUnitRef calls.
		if ( key )
		{
			ODID	 scopeFrameID = (scopeFrame ? scopeFrame->GetID(ev) : kODNULLID);
			ODDraft* fromDraft = fDependentFrame->GetDraft();
			frameID = fromDraft->WeakClone(ev, key, frameID, kODNULLID, scopeFrameID);
		}
		
		// Write out weak references to each of the part's display frames.
		ODSetWeakSURefProp(ev, storageUnit, kODNULL, kODNULL, frameID);
	}
	else
	{
		ODStorageUnitRef emptyRef = {0,0,0,0};
		StorageUnitSetValue(storageUnit, ev, kODStorageUnitRefSize, &emptyRef);
	}
}

//------------------------------------------------------------------------------
// CFrameInfo::CloneInto
//------------------------------------------------------------------------------

void CFrameInfo::CloneInto(Environment *ev, ODDraftKey key,
							ODStorageUnitView* storageUnitView, ODFrame* scopeFrame)
{
	// This method assumes that OpenDoc has passed us a storageUnitView
	// that is focused to a property, but no particular value.
	
	ODStorageUnit* storageUnit = storageUnitView->GetStorageUnit(ev);

	if ( !storageUnit->Exists(ev, kODNULL, kCappuccinoInfo, 0) )
	{
		this->ExternalizeFrameInfo(ev, storageUnit, key, scopeFrame);
	}
}

//------------------------------------------------------------------------------
// CFrameInfo::InitFromStorage
//------------------------------------------------------------------------------

void CFrameInfo::InitFromStorage(Environment* ev, ODStorageUnitView* storageUnitView)
{
	// This method assumes that OpenDoc has passed us a storageUnitView
	// that is focused to a property, but no particular value.
	
	ODStorageUnit* storageUnit = storageUnitView->GetStorageUnit(ev);

	if ( ODSUExistsThenFocus(ev, storageUnit, kODNULL, kCappuccinoInfo) )
	{
		TRY
			ODID frameID = ODGetWeakSURefProp(ev, storageUnit, kODNULL, kODNULL);
			if ( frameID != kODNULLID )
			{
				// Create a proxy class to support the lazy internalization.
				CFrameProxy* proxy = new CFrameProxy;
				proxy->InitFrameProxy(frameID, ODGetDraft(ev, storageUnit));

				// Store the proxy source frame.
				fSourceFrame = proxy;
			}
			
		CATCH_ALL
			ODDeleteObject(fSourceFrame);
			fSourceFrame = kODNULL;
		ENDTRY

		TRY
			ODID frameID = ODGetWeakSURefProp(ev, storageUnit, kODNULL, kODNULL);
			if ( frameID != kODNULLID )
			{
				// Create a proxy class to support the lazy internalization.
				CFrameProxy* proxy = new CFrameProxy;
				proxy->InitFrameProxy(frameID, ODGetDraft(ev, storageUnit));

				// Store the proxy dependent frame.
				fDependentFrame = proxy;
			}
			
		CATCH_ALL
			ODDeleteObject(fDependentFrame);
			fDependentFrame = kODNULL;
		ENDTRY
	}
}

//------------------------------------------------------------------------------
// CFrameInfo::SetSourceFrame
//------------------------------------------------------------------------------

void CFrameInfo::SetSourceFrame(Environment* ev, ODFrame* frame)
{
	if ( frame != kODNULL )
	{
		// Create a proxy class to support the lazy internalization.
		CFrameProxy* proxy = new CFrameProxy;
		proxy->InitFrameProxy(ev,frame);
		
		// Store the proxy source frame after clearing the old one.
		ODDeleteObject(fSourceFrame);
		fSourceFrame = proxy;
	}
}

//------------------------------------------------------------------------------
// CFrameInfo::ReleaseSourceFrame
//------------------------------------------------------------------------------

void CFrameInfo::ReleaseSourceFrame(Environment* ev)
{
	ODDeleteObject(fSourceFrame);
}

//------------------------------------------------------------------------------
// CFrameInfo::SetDependentFrame
//------------------------------------------------------------------------------

void CFrameInfo::SetDependentFrame(Environment* ev, ODFrame* frame)
{
	if ( frame != kODNULL )
	{
		// Create a proxy class to support the lazy internalization.
		CFrameProxy* proxy = new CFrameProxy;
		proxy->InitFrameProxy(ev,frame);
		
		// Store the proxy dependent frame after clearing the old one.
		ODDeleteObject(fDependentFrame);
		fDependentFrame = proxy;
	}
}

//------------------------------------------------------------------------------
// CFrameInfo::ReleaseDependentFrame
//------------------------------------------------------------------------------

void CFrameInfo::ReleaseDependentFrame(Environment* ev)
{
	ODDeleteObject(fDependentFrame);
}

//------------------------------------------------------------------------------
// CFrameInfo::AcquirePartWindow
//------------------------------------------------------------------------------

ODWindow* CFrameInfo::AcquirePartWindow(Environment* ev)
{
	ODWindow* window = kODNULL;
	
	if ( fPartWindowID )
	{
		window = fSession->GetWindowState(ev)->AcquireWindow(ev,fPartWindowID);
		if ( !window ) 
			fPartWindowID = kODNULLID;
	}
	
	return window;
}

//------------------------------------------------------------------------------
// CFrameInfo::SetPartWindow
//------------------------------------------------------------------------------

void CFrameInfo::SetPartWindow(Environment* ev, ODWindow* window)
{
	fPartWindowID = window ? window->GetID(ev) : kODNULLID;
}

//------------------------------------------------------------------------------
// CFrameInfo::GetFrameInfo (ODFrame)								[static]
//------------------------------------------------------------------------------

CFrameInfo* CFrameInfo::GetFrameInfo(Environment* ev, ODFrame* frame)
{
	ASSERT_NOT_NULL(frame);
	return (CFrameInfo*) frame->GetPartInfo(ev);
}

//------------------------------------------------------------------------------
// CFrameInfo::GetFrameInfo (ODFacet)								[static]
//------------------------------------------------------------------------------

CFrameInfo* CFrameInfo::GetFrameInfo(Environment* ev, ODFacet* facet)
{
	ASSERT_NOT_NULL(facet);
	return CFrameInfo::GetFrameInfo(ev, facet->GetFrame(ev));
}

