/*
	File:		som_DrawEditor.cpp

	Contains:	Sample Part's SOM based part public interface implementation

	Written by:	Dave Stafford
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.

	Notes:		Like SamplePart, DrawEditor is implemented using a "wrapped" class strategy.
				This means that the SOM class is just a smart delegator and
				the real implementation lives in a C++ class. The reason for
				doing this is twofold. First, this strategy all but elimintates
				the need for you to have to modify the IDL file and run the
				SOM compiler/emitter. Secondly, given that all code changes
				will occur in the C++ class, build turn around time is reduced
				to however fast the compiler can crunch code.
				
				This is not to say that implementing pure SOM classes isn't
				possible, nor necessary. There will be objects (ie. Extensions)
				which need to be exported for other parts to use which will need
				to be written in IDL and SOM implementation files.

*/

// Notification that this is a SOM source file
#define SampleCode_som_DrawEditor_Class_Source

// define underscore field names (ie. _fSelf)
#define VARIABLE_MACROS

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Special Includes --

#ifndef _EXCEPT_
#include <Except.h>
#endif

// -- DrawEditor Includes --

#ifndef SOM_SampleCode_som_DrawEditor_xih
#include "som_DrawEditor.xih"
#endif

#ifndef _DrawEditor_
#include "DrawEditor.h"
#endif

#ifndef SOM_EmbeddedFramesIterator_xh
#include "som_FrameIterator.xh"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

// -- OpenDoc Utilities --

#ifndef _ODUTILS_
#include "ODUtils.h"
#endif

#pragma segment somDrawEditor

SOM_Scope COrderedList*  SOMLINK som_DrawEditor__GetEmbeddedFrames(SampleCode_som_DrawEditor *somSelf, Environment *ev)
{
    SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
    SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__GetEmbeddedFrames");

    /* Return our list of embedded frames */
    return _fPart->GetEmbeddedFrames();
}

//====================================================================
// som_DrawEditor
//====================================================================

SOM_Scope void  SOMLINK som_DrawEditor__somInit(SampleCode_som_DrawEditor *somSelf)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__somInit");

	SampleCode_som_DrawEditor_parent_ODPart_somInit(somSelf);
	
	// somInit and somUninit methods behave like C++ constructors in that the
	// inherited methods are called automatically. Thus, there is no need to
	// call the parent class' somInit or somUninit.
	
	// There is also no need to set instance variables to zero/NULL
	// since SOM guarantees that a newly constructed object is zeroed.
	
	// Lastly, it is not valid to perform any operation here which may throw
	// an exception.
}

SOM_Scope void  SOMLINK som_DrawEditor__somUninit(SampleCode_som_DrawEditor *somSelf)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__somUninit");

	delete _fPart;

	SampleCode_som_DrawEditor_parent_ODPart_somUninit(somSelf);
}


SOM_Scope void  SOMLINK som_DrawEditor__InitPart(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	// We must call the initialize method of our parent
	// class to allow OpenDoc to annotate our part's storageUnit, to 
	// set our refcount, and to change our "initialized" flag to true.
	SampleCode_som_DrawEditor_parent_ODPart_InitPart(somSelf, ev, storageUnit, partWrapper);
	
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__InitPart");
	
	SOM_TRY
		// Create the "real" C++ DrawEditor.		
		_fPart = new DrawEditor;
	
		_fPart->InitPart(ev,storageUnit,partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY


}

SOM_Scope void  SOMLINK som_DrawEditor__InitPartFromStorage(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	// Call the initialize method of our parent PersistentObject class to generate
	// the appropriate time/date stamp on our ODPart's storage unit.
	SampleCode_som_DrawEditor_parent_ODPart_InitPartFromStorage(somSelf, ev, storageUnit, partWrapper);
	
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__InitPartFromStorage");

	SOM_TRY
		// Create the "real" C++ DrawEditor.		
		_fPart = new DrawEditor;
		
		_fPart->InitPartFromStorage(ev,storageUnit,partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY

}


SOM_Scope ODSize  SOMLINK som_DrawEditor__Purge(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODSize size)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__Purge");

	// Call the helper "Part" object and return the result
	ODSize tSize = 0;
	ODVolatile(tSize);
	
	SOM_TRY
		tSize = _fPart->Purge(ev,size);
	SOM_CATCH_ALL
		return tSize;
	SOM_ENDTRY

	return tSize;
}

SOM_Scope void  SOMLINK som_DrawEditor__Externalize(SampleCode_som_DrawEditor *somSelf, Environment *ev)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__Externalize");

	SampleCode_som_DrawEditor_parent_ODPart_Externalize(somSelf,ev);
	
	// Call the helper object. If failure occurs, the ev parameter is set automatically.
	SOM_TRY
		_fPart->Externalize(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
}

SOM_Scope void  SOMLINK som_DrawEditor__Release(SampleCode_som_DrawEditor *somSelf, Environment *ev)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__Release");

	SampleCode_som_DrawEditor_parent_ODPart_Release(somSelf,ev);
	
	SOM_TRY
		_fPart->Release(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__ReleaseAll(SampleCode_som_DrawEditor *somSelf, Environment *ev)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__ReleaseAll");

	SOM_TRY
		_fPart->ReleaseAll(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
	SampleCode_som_DrawEditor_parent_ODPart_ReleaseAll(somSelf,ev);
}


SOM_Scope ODBoolean  SOMLINK som_DrawEditor__HasExtension(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODType extensionName)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__HasExtension");

	ODBoolean result;

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
		result = SampleCode_som_DrawEditor_parent_ODPart_HasExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_DrawEditor__ReleaseExtension(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODExtension* extension)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__ReleaseExtension");

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
	    SampleCode_som_DrawEditor_parent_ODPart_ReleaseExtension(somSelf,ev,extension);
	SOM_CATCH_ALL
	SOM_ENDTRY
}


SOM_Scope ODExtension*  SOMLINK som_DrawEditor__AcquireExtension(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODType extensionName)
{
    SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
    SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__AcquireExtension");

	ODExtension* result;

	SOM_TRY
		// This part doesn't support any extensions, but one of our parent classes might;
		result = SampleCode_som_DrawEditor_parent_ODPart_AcquireExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_DrawEditor__FulfillPromise(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODStorageUnitView* promiseSUView)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__Release");

	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__FulfillPromise");

	SOM_TRY
		_fPart->FulfillPromise(ev,promiseSUView);
	SOM_CATCH_ALL
	SOM_ENDTRY

}


SOM_Scope void  SOMLINK som_DrawEditor__DropCompleted(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODPart* part,
		ODDropResult dropResult)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__DropCompleted");
	
	SOM_TRY
		_fPart->DropCompleted(ev,part, dropResult);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
}

SOM_Scope ODDragResult  SOMLINK som_DrawEditor__DragEnter(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODDragItemIterator* dragInfo,
		ODFacet* facet,
		ODPoint* where)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__DragEnter");

	// Call the helper object and return the result. If the helper fails, kODDropFail should 
	// be returned
	
	// Call Helper and return result.
	ODDragResult tDragResult = kODFalse;
	
	SOM_TRY
		tDragResult = _fPart->DragEnter(ev,dragInfo, facet, where);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
	return tDragResult;

}

SOM_Scope ODDragResult  SOMLINK som_DrawEditor__DragWithin(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODDragItemIterator* dragInfo,
		ODFacet* facet,
		ODPoint* where)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__DragWithin");

	// Call Helper and return result.
	ODDragResult tDragResult = kODFalse;
	
	SOM_TRY
		tDragResult = _fPart->DragWithin(ev, dragInfo, facet, where);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
	return tDragResult;

}

SOM_Scope void  SOMLINK som_DrawEditor__DragLeave(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFacet* facet,
		ODPoint* where)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__DragLeav");

	// Call helper.
	SOM_TRY
		_fPart->DragLeave(ev, facet, where);
	SOM_CATCH_ALL
	SOM_ENDTRY


}

SOM_Scope ODDropResult  SOMLINK som_DrawEditor__Drop(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODDragItemIterator* dropInfo,
		ODFacet* facet,
		ODPoint* where)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__Drop");
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);

	ODDropResult tDropResult = kODDropFail;
	
	// Call helper.
	SOM_TRY
		tDropResult = _fPart->Drop(ev, dropInfo, facet, where);
	SOM_CATCH_ALL
	SOM_ENDTRY


	return tDropResult;
}


SOM_Scope void  SOMLINK som_DrawEditor__ContainingPartPropertiesUpdated(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnit* propertyUnit)
{
    // SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
    SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__ContainingPartPropertiesUpdated");

    SampleCode_som_DrawEditor_parent_ODPart_ContainingPartPropertiesUpdated(somSelf,ev,frame,propertyUnit);
}

SOM_Scope ODStorageUnit*  SOMLINK som_DrawEditor__AcquireContainingPartProperties(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
    // SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
    SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__AcquireContainingPartProperties");

    return (SampleCode_som_DrawEditor_parent_ODPart_AcquireContainingPartProperties(somSelf,ev,frame));
}

SOM_Scope ODBoolean  SOMLINK som_DrawEditor__RevealFrame(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODShape* /*revealShape*/)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__RevealFrame");

	// DCS $$$$$
	DebugStr("\pmust implement this.");
	
	return kODFalse;
}

SOM_Scope void  SOMLINK som_DrawEditor__EmbeddedFrameSpec(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODObjectSpec* /*spec*/)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__EmbeddedFrameSpec");

	// DCS $$$$$
	DebugStr("\pmust implement this.");
}

SOM_Scope ODEmbeddedFramesIterator*  SOMLINK som_DrawEditor__CreateEmbeddedFramesIterator(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__CreateEmbeddedFramesIterator");

	som_EmbeddedFramesIterator* tIter = kODNULL;
	ODVolatile(tIter);
	
	SOM_TRY
		tIter = new som_EmbeddedFramesIterator;
		tIter->InitEmbeddedFramesIterator(ev, somSelf);
	SOM_CATCH_ALL
		ODDeleteObject(tIter);
	SOM_ENDTRY

	
	return tIter;
}

SOM_Scope void  SOMLINK som_DrawEditor__DisplayFrameAdded(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__DisplayFrameAdded");

	SOM_TRY
		_fPart->DisplayFrameAdded(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__DisplayFrameRemoved(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__DisplayFrameRemoved");

	SOM_TRY
		_fPart->DisplayFrameRemoved(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__DisplayFrameConnected(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__DisplayFrameConnected");

	SOM_TRY
		_fPart->DisplayFrameConnected(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__DisplayFrameClosed(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__DisplayFrameClosed");

	SOM_TRY
		_fPart->DisplayFrameClosed(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__AttachSourceFrame(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODFrame* sourceFrame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__AttachSourceFrame");

	SOM_TRY
		_fPart->AttachSourceFrame(ev,frame,sourceFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__FrameShapeChanged(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__FrameShapeChanged");

	SOM_TRY
		_fPart->FrameShapeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__ViewTypeChanged(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__ViewTypeChanged");

	SOM_TRY
		_fPart->ViewTypeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__PresentationChanged(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__PresentationChanged");

	// DCS $$$$$
	DebugStr("\pmust implement this.");
}

SOM_Scope void  SOMLINK som_DrawEditor__SequenceChanged(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__SequenceChanged");

	// DCS $$$$$
	DebugStr("\pmust implement this.");
}

SOM_Scope void  SOMLINK som_DrawEditor__WritePartInfo(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__WritePartInfo");

	SOM_TRY
		_fPart->WritePartInfo(ev,partInfo,storageUnitView);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODInfoType  SOMLINK som_DrawEditor__ReadPartInfo(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnitView* storageUnitView)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__ReadPartInfo");

	ODInfoType tInfo = kODNULL;
	
	SOM_TRY
		tInfo = _fPart->ReadPartInfo(ev,frame,storageUnitView);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
	return tInfo;
}

SOM_Scope ODID  SOMLINK som_DrawEditor__Open(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__Open");

	ODID tID = kODNULLID;
	
	SOM_TRY
		tID = _fPart->Open(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY


	return tID;
}


SOM_Scope ODFrame*  SOMLINK som_DrawEditor__RequestEmbeddedFrame(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* containingFrame,
		ODFrame* baseFrame,
		ODShape* frameShape,
		ODPart* embedPart,
		ODTypeToken viewType,
		ODTypeToken presentation,
		ODBoolean isOverlaid)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__RequestEmbeddedFrame");

	ODFrame* tFrame = kODNULL;
	
	SOM_TRY
		tFrame = _fPart->RequestEmbeddedFrame(ev, containingFrame, 
									baseFrame, frameShape, embedPart, viewType, presentation, isOverlaid);
	SOM_CATCH_ALL
		return tFrame;
	SOM_ENDTRY


	return tFrame;
}

SOM_Scope void  SOMLINK som_DrawEditor__RemoveEmbeddedFrame(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* embeddedFrame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__RemoveEmbeddedFrame");

	SOM_TRY
		_fPart->RemoveEmbeddedFrame(ev, embeddedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY


}

SOM_Scope ODShape*  SOMLINK som_DrawEditor__RequestFrameShape(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* embeddedFrame,
		ODShape* frameShape)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__RequestFrameShape");

	
	ODShape* tShape = kODNULL;
	
	SOM_TRY
		tShape = _fPart->RequestFrameShape(ev, embeddedFrame, frameShape);
	SOM_CATCH_ALL
	SOM_ENDTRY

	return tShape;
}

SOM_Scope void  SOMLINK som_DrawEditor__UsedShapeChanged(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* embeddedFrame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__UsedShapeChanged");

	SOM_TRY
		_fPart->UsedShapeChanged(ev, embeddedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY


}

SOM_Scope ODShape*  SOMLINK som_DrawEditor__AdjustBorderShape(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFacet* embeddedFacet,
		ODShape* shape)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__AdjustBorderShape");

	ODShape* tShape = kODNULL;
	
	SOM_TRY
		tShape = _fPart->AdjustBorderShape(ev, embeddedFacet, shape);
	SOM_CATCH_ALL
	SOM_ENDTRY

	return tShape;
}

SOM_Scope void  SOMLINK som_DrawEditor__FacetAdded(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFacet* facet)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__FacetAdded");

	SOM_TRY
		_fPart->FacetAdded(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
}

SOM_Scope void  SOMLINK som_DrawEditor__FacetRemoved(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFacet* facet)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__FacetRemoved");

	SOM_TRY
		_fPart->FacetRemoved(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__CanvasChanged(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFacet* /*facet*/)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__CanvasChanged");

	// Do I need to implement this? $$$$$ DCS
}

SOM_Scope void  SOMLINK som_DrawEditor__GeometryChanged(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFacet* facet,
		ODBoolean clipShapeChanged,
		ODBoolean externalTransformChanged)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__GeometryChanged");

	SOM_TRY
		_fPart->GeometryChanged(ev,facet,clipShapeChanged,externalTransformChanged);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__Draw(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFacet* facet,
		ODShape* invalidShape)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__Draw");

	SOM_TRY
		_fPart->Draw(ev,facet,invalidShape);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__CanvasUpdated(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODCanvas* /*canvas*/)
{

	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__CanvasUpdated");

	// Do I need to implement this? $$$$$ DCS
}

SOM_Scope void  SOMLINK som_DrawEditor__HighlightChanged(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFacet* facet)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__HighlightChanged");

	SOM_TRY
		_fPart->HighlightChanged(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope ODULong  SOMLINK som_DrawEditor__GetPrintResolution(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	//SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__GetPrintResolution");

	// 72 is the default Quickdraw resolution and also OpenDoc's default canvas resolution.
	return 72;
}

SOM_Scope ODLinkSource*  SOMLINK som_DrawEditor__CreateLink(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODByteArray* data)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__CreateLink");

	ODLinkSource *linkSource = kODNULL;
	SOM_TRY
		linkSource = _fPart->CreateLink(ev, data);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
	return linkSource;
}


SOM_Scope void  SOMLINK som_DrawEditor__LinkUpdated(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODLink* odLink,
		ODUpdateID change)
{
    SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__LinkUpdated");

	SOM_TRY
		_fPart->LinkUpdated(ev, odLink, change);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__RevealLink(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODLinkSource* linkSource)
{
    SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__RevealLink");

	SOM_TRY
		_fPart->RevealLink(ev, linkSource);
	SOM_CATCH_ALL
	SOM_ENDTRY


}


SOM_Scope ODBoolean  SOMLINK som_DrawEditor__EditInLinkAttempted(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__EditInLinkAttempted");

	ODBoolean result = kODFalse;
	SOM_TRY
		result = _fPart->EditInLinkAttempted(ev, frame);
	SOM_CATCH_ALL
		return result;
	SOM_ENDTRY

	
	return result;
}


SOM_Scope void  SOMLINK som_DrawEditor__EmbeddedFrameUpdated(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODUpdateID change)
{
    SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
    SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__EmbeddedFrameUpdated");

	SOM_TRY
		_fPart->EmbeddedFrameUpdated(ev,frame,change);
	SOM_CATCH_ALL
	SOM_ENDTRY


}

SOM_Scope void  SOMLINK som_DrawEditor__LinkStatusChanged(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__LinkStatusChanged");
    SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);

	SOM_TRY
		_fPart->LinkStatusChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODBoolean  SOMLINK som_DrawEditor__BeginRelinquishFocus(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__BeginRelinquishFocus");

	ODBoolean tBoolean = kODFalse;
	
	SOM_TRY
		tBoolean = _fPart->BeginRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY

	return tBoolean;
}

SOM_Scope void  SOMLINK som_DrawEditor__CommitRelinquishFocus(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__CommitRelinquishFocus");

	SOM_TRY
		_fPart->CommitRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
}

SOM_Scope void  SOMLINK som_DrawEditor__AbortRelinquishFocus(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__AbortRelinquishFocus");

	SOM_TRY
		_fPart->AbortRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY

	
}

SOM_Scope void  SOMLINK som_DrawEditor__FocusAcquired(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__FocusAcquired");

	SOM_TRY
		_fPart->FocusAcquired(ev,focus,ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__FocusLost(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__FocusLost");

	SOM_TRY
		_fPart->FocusLost(ev,focus,ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__CloneInto(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODDraftKey key,
		ODStorageUnit* toSU,
		ODFrame* scope)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__CloneInto");

	SampleCode_som_DrawEditor_parent_ODPart_CloneInto(somSelf,ev,key,toSU,scope);

	SOM_TRY
		_fPart->CloneInto(ev,key,toSU,scope);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__ExternalizeKinds(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODTypeList* kindset)
{

	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__ExternalizeKinds");

	SOM_TRY
		_fPart->ExternalizeKinds(ev,kindset);
	SOM_CATCH_ALL
	SOM_ENDTRY
}


SOM_Scope void  SOMLINK som_DrawEditor__ChangeKind(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODType kind)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__ChangeKind");

	SOM_TRY
		_fPart->ChangeKind(ev,kind);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_DrawEditor__ClonePartInfo(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODDraftKey key,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView,
		ODFrame* scope)
{
    SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
    SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__ClonePartInfo");

	SOM_TRY
		_fPart->ClonePartInfo(ev,key,partInfo,storageUnitView,scope);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope ODBoolean  SOMLINK som_DrawEditor__HandleEvent(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODEventData* event,
		ODFrame* frame,
		ODFacet* facet,
		ODEventInfo* eventInfo)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__HandleEvent");

	ODBoolean handled = kODFalse;
	
	SOM_TRY
		handled = _fPart->HandleEvent(ev,event,frame,facet,eventInfo);
	SOM_CATCH_ALL
	SOM_ENDTRY

	return handled;
}

SOM_Scope void  SOMLINK som_DrawEditor__AdjustMenus(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__AdjustMenus");

	SOM_TRY
		_fPart->AdjustMenus(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__UndoAction(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODActionData* actionState)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__UndoAction");

	SOM_TRY
		_fPart->UndoAction(ev,actionState);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__RedoAction(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODActionData* actionState)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__RedoAction");

	SOM_TRY
		_fPart->RedoAction(ev,actionState);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__DisposeActionState(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODActionData* actionState,
		ODDoneState doneState)
{
	SampleCode_som_DrawEditorData *somThis = SampleCode_som_DrawEditorGetData(somSelf);
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__DisposeActionState");

	SOM_TRY
		_fPart->DisposeActionState(ev,actionState, doneState);
	SOM_CATCH_ALL
	SOM_ENDTRY

}

SOM_Scope void  SOMLINK som_DrawEditor__WriteActionState(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODActionData* /*actionState*/,
		ODStorageUnitView* /*storageUnitView*/)
{
	// This is for persistent undo which we is not currently supported.
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__WriteActionState");
}

SOM_Scope ODActionData  SOMLINK som_DrawEditor__ReadActionState(SampleCode_som_DrawEditor *somSelf, Environment *ev,
		ODStorageUnitView* /*storageUnitView*/)
{
	SampleCode_som_DrawEditorMethodDebug("SampleCode_som_DrawEditor","som_DrawEditor__ReadActionState");

	ODActionData data;
	data._maximum = 0;
	data._length = 0;
	data._buffer = kODNULL;
	return ((ODActionData)data);
}
