/*
	File:		ListItem.cpp

	Contains:	List Item Class Implementation

	Written by:	Steve Smith
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (ie. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with the "ev" parameter set.
#include <Except.h>
#endif

// -- PanelEditor Includes --

#ifndef _LISTITEM_
#include "ListItem.h"
#endif

#ifndef _PANELEDITORDEF_
#include "PanelEditorDef.h"
#endif

#ifndef _PANELEDITORGLOBALS_
#include "PanelEditorGlobals.h"
#endif

#ifndef _PANELEDITORUTILS_
#include "PanelEditorUtils.h"
#endif

#ifndef _SAMPLECOLLECTIONS_
#include "SampleCollections.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef SOM_ODDraft_xh
#include <Draft.xh>
#endif

#ifndef SOM_ODInfo_xh
#include <Info.xh>
#endif

// -- OpenDoc Utilities --

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef _FOCUSLIB_
#include <FocusLib.h>
#endif

#ifndef _STDTYPIO_
#include <StdTypIO.h>
#endif

#ifndef _ITEXT_
#include <IText.h>
#endif

#ifndef _STORUTIL_
#include <StorUtil.h>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef _ODNEW_
#include <ODNew.h>
#endif

#ifndef _TEMPOBJ_
#include <TempObj.h>
#endif

#ifndef _TEMPITER_
#include <TempIter.h>
#endif

#ifndef _ODDEBUG_
#include <ODDebug.h>
#endif

// -- Toolbox Includes --

#ifndef __TEXTUTILS__
#include <TextUtils.h>
#endif

#ifndef __QUICKDRAW__
#include <QuickDraw.h>
#endif

#pragma segment PanelEditorListItem

//==============================================================================
// CEmbeddedFrameProxy
//==============================================================================

//------------------------------------------------------------------------------
// Method:		Constructor
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

CEmbeddedFrameProxy::CEmbeddedFrameProxy()
{
	fFrame = kODNULL;
	fContainingFrame = kODNULL;
	fInited = kODFalse;
}

//------------------------------------------------------------------------------
// Method:		InitEmbeddedFrameProxy
//
// Description:	Perform all constructor operations that might fail.
//------------------------------------------------------------------------------

void CEmbeddedFrameProxy::InitEmbeddedFrameProxy(Environment* ev, ODFrame* frame)
{
	ODAcquireObject(ev, frame);
	fFrame = frame;
	fContainingFrame = kODNULL;
	fInited = kODTrue;
}

//------------------------------------------------------------------------------
// Method:		Constructor
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

CEmbeddedFrameProxy::~CEmbeddedFrameProxy()
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

#if ODDebug
	if ( fFrame || fContainingFrame )
		DebugStr("\pForgot To Cleanup");
#endif

	ODSafeReleaseObject(fFrame);
	ODSafeReleaseObject(fContainingFrame);
}

//------------------------------------------------------------------------------
// Method:		Constructor
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

void CEmbeddedFrameProxy::Release(Environment* ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	// If the containing frame field is set, then the frame has been
	// detached from the document frame hierarchy. If the proxy is being
	// deleted with the frame still detached, then we need to dispose of
	// the frame.
	if ( fContainingFrame )
	{
		fFrame->Remove(ev);
		fFrame = kODNULL;
	}
	// Release the proxied frame.
	ODReleaseObject(ev, fFrame);
	// Release the containing frame, if necessary.
	ODReleaseObject(ev, fContainingFrame);
}

//------------------------------------------------------------------------------
// Method:		Constructor
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

ODFrame* CEmbeddedFrameProxy::GetFrame()
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	return fFrame;
}

//------------------------------------------------------------------------------
// Method:		Constructor
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

void CEmbeddedFrameProxy::SetFrame(Environment* ev, ODFrame* frame)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	ODAcquireObject(ev, frame);
	ODReleaseObject(ev, fFrame);
	fFrame = frame;
}

//------------------------------------------------------------------------------
// Method:		GetContainingFrame
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

ODFrame* CEmbeddedFrameProxy::GetContainingFrame()
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	return fContainingFrame;
}

//------------------------------------------------------------------------------
// Method:		SetContainingFrame
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

void CEmbeddedFrameProxy::SetContainingFrame(Environment* ev, ODFrame* frame)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	ODAcquireObject(ev, frame);
	ODReleaseObject(ev, fContainingFrame);
	fContainingFrame = frame;
}


//==============================================================================
// CListItem
//==============================================================================

//------------------------------------------------------------------------------
// Method:		Constructor
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

CListItem::CListItem()
{
	fSelected = kODFalse;
	fName = kODNULL;
	fFrameProxies = kODNULL;
	fRefCount = 1;
	fPosition = 0;
	fInvalRgn = kODNULL;
	fNameRect.top = 0;
	fNameRect.left = 0;
	fNameRect.bottom = 0;
	fNameRect.right = 0;
	fPart = kODNULL;
	fInited = kODFalse;	
}

//------------------------------------------------------------------------------
// Method:		InitListItem
//
// Description:	Perform all constructor operations that might fail.
//------------------------------------------------------------------------------

void CListItem::InitListItem()
{
	fFrameProxies = new CList;
	fInvalRgn = ODNewRgn();
	fInited = kODTrue;
}

//------------------------------------------------------------------------------
// Method:		~CListItem
//
// Description:	This method is called during
//------------------------------------------------------------------------------

CListItem::~CListItem()
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fRefCount == 0, kAssertionFailed);
}

//------------------------------------------------------------------------------
// Method:		Acquire
//
// Description:	Public method for other objects to acquire a reference to the
//				item.
//------------------------------------------------------------------------------

void CListItem::Acquire()
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	fRefCount++;
}

//------------------------------------------------------------------------------
// Method:		Release
//
// Description:	Public method for other objects to release a previously acquired
//				reference. When the "count" drops to zero, the item is
//				automatically garbage collected.
//------------------------------------------------------------------------------

void CListItem::Release(Environment* ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fRefCount != 0, kAssertionFailed);

	fRefCount--;

	if ( fRefCount == 0 )
	{
		this->ReleaseAll(ev);
		delete this;
	}
}

//------------------------------------------------------------------------------
// Method:		ReleaseAll
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::ReleaseAll(Environment* ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fName != kODNULL, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		iter.RemoveCurrent();
		proxy->Release(ev);
		delete proxy;
	}
	
	ODDeleteObject(fFrameProxies);
	DisposeIText(fName);
	ODDisposeHandle((ODHandle)fInvalRgn);
	ODReleaseObject(ev, fPart);
}

//------------------------------------------------------------------------------
// Method:		SetName
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::SetName(Environment* ev, ODIText* name)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fName != kODNULL, kAssertionFailed);

	if ( name && fName )
		DisposeIText(fName);
		
	fName = name;
}

//------------------------------------------------------------------------------
// Method:		GetName
//
// Description:	This method is called during
//------------------------------------------------------------------------------

ODIText* CListItem::GetName(Environment* ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	if ( (fName == kODNULL) || (GetITextStringLength(fName) == 0) )
		this->RetrieveName(ev);
	
	return fName;
}

//------------------------------------------------------------------------------
// Method:		RetrieveName
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::RetrieveName(Environment* ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	TempODPart part = this->AcquirePart(ev);
	
	if ( part )
	{
		TRY
			ODIText* newName = ODGetITextProp(ev, part->GetStorageUnit(ev),
											kODPropName, kODMacIText, kODNULL);
			
			if ( newName )
			{
				DisposeIText(fName);
				fName = newName;
			}
		CATCH_ALL
			// exception consumed
		ENDTRY
	}
}

//------------------------------------------------------------------------------
// Method:		CalcNameRect
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::CalcNameRect()
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	// Change the port font,size,style 
	CDrawingText localizePort(fName);

	// Calculate where to start drawing.
	ODUShort start = fItemOffset.v;
	start += (kNameGapMargin+kODLargeIconSize);
	
	// Setup an invalidation region based on the current rect being
	// drawn into.
	RectRgn(fInvalRgn, &fNameRect);
	
	// Give the name a large box to draw in.
	SetRect(&fNameRect, kNameGapMargin+2, start,
					kListWidth-17, start+kNameBoxHeight);
	
	// Get the name.
	char* string = GetITextCString(fName,kODNULL);
	short stringlen = GetITextStringLength(fName);
	
	// If the name is blank, we need to grab the "untitled"
	// name for the item.
	if ( stringlen == 0 )
	{
		// Dispose of the "old" blank name.
		DisposeIText(fName);

		// Get the "untitled" name.
		fName = CopyIText(gGlobals->fUntitledName);
		
		string = GetITextCString(fName,kODNULL);
		stringlen = GetITextStringLength(fName);
	}
	
	// Constrain the box based on the text width for the given
	// name drawn in the part's script and font.
	ODUShort width = TextWidth(string, 0, stringlen);
								
	// Calculate the actual difference between the string width (including a bit of
	// padding for highlighting) and the rect we've set up for the name.
	ODSShort diff = (fNameRect.right-fNameRect.left) - (width + 2*kNameHighlightPadding);

	// Inset the rect if the width of the name is narrower
	// than the box we are going to draw it in.
	if ( diff > 0 )
	{
		InsetRect(&fNameRect, ((ODUShort)diff/2), 0);
	}
	// Otherswise, we need to truncate the name because
	// it's too wide.
	else
	{
		TruncText((fNameRect.right-fNameRect.left)-4, string, &stringlen, truncMiddle);
		string[stringlen] = 0x00;
		SetITextCString(fName, string);
	}
	
	delete string;
	
	// Subtract the "new" area from invalidation region.
	// (Note: the invalid region is used when drawing.)
	RgnHandle subtRgn = NewRgn();
	RectRgn(subtRgn, &fNameRect);
	XorRgn(fInvalRgn, subtRgn, fInvalRgn);
	DisposeRgn(subtRgn);
}

//------------------------------------------------------------------------------
// Method:		SetListPosition
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::SetListPosition(Environment* ev, ODUShort pos)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	// Calcuate the new position for the item.
	fItemOffset.h = kListLeftMargin;
	fItemOffset.v = pos*kListItemHeight+kListTopMargin;
	
	fPosition = pos;
	
	// Update the positional information related to drawing
	// the embedded items name.
	if ( fName )
		this->CalcNameRect();
	// Verify that the embedded item's new position is in sync
	// with its frame and facets geometry.
	this->Check(ev);
}

//------------------------------------------------------------------------------
// Method:		GetListPosition
//
// Description:	This method is called during
//------------------------------------------------------------------------------

ODUShort CListItem::GetListPosition()
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	return fPosition;
}

//------------------------------------------------------------------------------
// Method:		Check
//
// Description: Public method to validate that the list item's position matches
//				exactly the external transform for the facets of the represented
//				frames.
//------------------------------------------------------------------------------

void CListItem::Check(Environment* ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy=(CEmbeddedFrameProxy*)iter.First();iter.IsNotComplete();
			 proxy=(CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODFrameFacetIterator iter2(ev, frame);
		TRY
			for ( ODFacet* facet = iter2.First(); iter2.IsNotComplete();
					facet = iter2.Next() )
			{
				TempODTransform extTransform = facet->CreateTransform(ev);
				
				extTransform->SetQDOffset(ev, &fItemOffset);
				
				facet->ChangeGeometry(ev, kODNULL, extTransform, kODNULL);
				facet->Invalidate(ev, kODNULL, kODNULL);
			}
		CATCH_ALL
			// exception consumed
		ENDTRY
	}
}

//------------------------------------------------------------------------------
// Method:		DrawAll
//
// Description:	Private method to affect redrawing of all frames associated with
//				this item. This is caused by signicant changes like highlight or
//				selection changes.
//------------------------------------------------------------------------------

void CListItem::DrawAll(Environment *ev, ODBoolean erase)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODShape usedShape = ODCopyAndRelease(ev, frame->AcquireUsedShape(ev, kODNULL));

		TempODFrameFacetIterator iter2(ev, frame);
		TRY
			for ( ODFacet* facet = iter2.First(); iter2.IsNotComplete();
					facet = iter2.Next() )
			{
				// Force the part to draw immediately, so the user gets
				// instant feedback.
				facet->Draw(ev, usedShape, kODNULL);
				facet->Validate(ev, usedShape, kODNULL);
	
				// Draw name.
				this->FocusAndDraw(ev, facet, erase);
			}
		CATCH_ALL
			// exception consumed. incomplete drawing is not fatal.
		ENDTRY
	}
}

//------------------------------------------------------------------------------
// Method:		Draw
//
// Description:	Public method to affect drawing of the embedded item's title.
//------------------------------------------------------------------------------

void CListItem::Draw(Environment* ev, ODFacet* containingFacet)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy=(CEmbeddedFrameProxy*)iter.First();iter.IsNotComplete();
			 proxy=(CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();

		TempODFrameFacetIterator iter2(ev, frame);
		for ( ODFacet* facet = iter2.First(); iter2.IsNotComplete();
				facet = iter2.Next() )
		{
			if ( ODObjectsAreEqual(ev, facet->GetContainingFacet(ev), containingFacet) )
			{
				this->FocusAndDraw(ev, facet, kODTrue);
			}
		}
	}
}

//------------------------------------------------------------------------------
// Method:		FocusAndDraw
//
// Description:	Private method to set up the drawing environment, find the
//				"active" state of the frame being drawn, and draw the title.
//------------------------------------------------------------------------------

void CListItem::FocusAndDraw(Environment *ev, ODFacet* facet, ODBoolean erase)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	CFocusContent initiateDrawing(ev, facet->GetContainingFacet(ev));
	ODBoolean active = facet->GetWindow(ev)->IsActive(ev);
	
	this->DrawName(active, erase);
}

//------------------------------------------------------------------------------
// Method:		DrawName
//
// Description:	Private method to draw the title of the item.
//------------------------------------------------------------------------------

void CListItem::DrawName(ODBoolean active, ODBoolean erase)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	// If the name recently changed, their might be some "leftovers".
	if ( !EmptyRgn(fInvalRgn) )
	{
		EraseRgn(fInvalRgn);
		SetEmptyRgn(fInvalRgn);
	}
		
	// Erase the area where the name.
	if ( erase )
		EraseRect(&fNameRect);

	CDrawingText localizePort(fName);
	TextMode(srcOr);

	// Draw the name.
	if ( fName != kODNULL )
	{
		if ( GetITextStringLength(fName) > 0 )
		{
			TETextBox(GetITextCString(fName,kODNULL), GetITextStringLength(fName),
							&fNameRect, teCenter);
		}
	}

	// If the item is selected, invert the name.
	// If the item is in an inactive window, frame it instead.
	if ( this->IsSelected() )
	{
		if ( active ) InvertRect(&fNameRect);
		else FrameRect(&fNameRect);
	}
}

//------------------------------------------------------------------------------
// Method:		Invalidate
//
// Description:	Public method to invalidate all embedded frames maintained by
//				this item.
//------------------------------------------------------------------------------

void CListItem::Invalidate(Environment* ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		frame->Invalidate(ev, kODNULL, kODNULL);
	}
}

//------------------------------------------------------------------------------
// Method:		AcquirePart
//
// Description:	Private method to acquire a reference to the part this item is
//				a proxy for.
//------------------------------------------------------------------------------

ODPart* CListItem::AcquirePart(Environment *ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);
	
	if ( !fPart )
	{
		CListIterator iter(fFrameProxies);
		CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
		ODFrame* frame = proxy->GetFrame();
		
		if ( frame ) fPart = frame->AcquirePart(ev);
	}
	
	ODAcquireObject(ev, fPart);	
	return fPart;
}

//------------------------------------------------------------------------------
// Method:		DoPartInfo
//
// Description:	Public method to handle a users request to edit the part info of
//				the item.
//------------------------------------------------------------------------------

void CListItem::DoPartInfo(Environment* ev, ODFrame* containingFrame)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	// Make sure our part field is set so we can use it to get the session.
	{ TempODPart part = this->AcquirePart(ev); }

	ODInfo* info = ODGetSession(ev, fPart)->GetInfo(ev);
	ODFacet* facet = kODNULL;

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODFrame framesContFrame = frame->AcquireContainingFrame(ev);
		
		if ( ODObjectsAreEqual(ev, framesContFrame, containingFrame) )
		{
			TempODFrameFacetIterator iter2(ev, frame);
			for ( facet = iter2.First(); iter2.IsNotComplete();
					facet = iter2.Next() )
			{
				if ( facet->IsSelected(ev) )
					break;
			}
			break;
		}
	}
	
	if ( facet )
	{
#ifndef PE_ViewerBuild
		if ( info->ShowPartFrameInfo(ev, facet, kODTrue) )
#else
		if ( info->ShowPartFrameInfo(ev, facet, kODFalse) )
#endif
		{
#ifndef PE_ViewerBuild
			// For now, OpenDoc does give parts enough control over the "info"
			// dialog. In particular, a user can change the view type, which
			// we don't want. So to keep things working, we always reset the
			// view type back to frame, if necessary.
			if ( facet->GetFrame(ev)->GetViewType(ev) != gGlobals->fLargeIconView )
				facet->GetFrame(ev)->ChangeViewType(ev, gGlobals->fLargeIconView);
#endif
			this->RetrieveName(ev);
			this->CalcNameRect();
			this->FocusAndDraw(ev, facet, kODTrue);
		}
	}
}

//------------------------------------------------------------------------------
// Method:		HitTest
//
// Description:	This method is called during
//------------------------------------------------------------------------------

ODBoolean CListItem::HitTest(Environment *ev, ODFrame* frame, Point pt)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	if ( frame )
	{
		CListIterator iter(fFrameProxies);
		for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
				iter.IsNotComplete();
				proxy = (CEmbeddedFrameProxy*)iter.Next() )
		{
			if ( ODObjectsAreEqual(ev, proxy->GetFrame(), frame) ) 
				return kODTrue;
		}
	}

	if ( PtInRect(pt, &fNameRect) )
		return kODTrue;
	
	return kODFalse;
}

//------------------------------------------------------------------------------
// Method:		IsSelected
//
// Description:	Return the selected state of the proxied item.
//------------------------------------------------------------------------------

ODBoolean CListItem::IsSelected() const
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	return fSelected;
}

//------------------------------------------------------------------------------
// Method:		SetSelected
//
// Description:	Change the selection and highlight state for the proxied item.
//------------------------------------------------------------------------------

void CListItem::SetSelected(Environment *ev, ODBoolean selected)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODWindow window = frame->AcquireWindow(ev);

		// Check for a valid window. If this item has been cut from
		// the document, it won't have a window.
		if ( window != kODNULL )
		{
			ODBoolean active = window->IsActive(ev);
		
			fSelected = selected;
	
			TempODFrameFacetIterator iter2(ev, frame);
			for ( ODFacet* facet = iter2.First(); iter2.IsNotComplete();
					facet = iter2.Next() )
			{
				ODHighlight highlight;
	
				if ( fSelected && active ) highlight = kODFullHighlight;
				else if ( fSelected && !active ) highlight = kODDimHighlight;
				else highlight = kODNoHighlight;
				
				facet->SetSelected(ev, fSelected);
				facet->ChangeHighlight(ev, highlight);
			}
		}
	}
	
	this->DrawAll(ev, kODTrue);
}

//------------------------------------------------------------------------------
// Method:		ActivationChanged
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::ActivationChanged(Environment* ev, ODID windowID, ODBoolean activate)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODWindow window = frame->AcquireWindow(ev);

		// Check for a valid window. If this item has been cut from
		// the document, it won't have a window.
		if ( window != kODNULL && window->GetID(ev) == windowID )
		{
			TempODFrameFacetIterator iter2(ev, frame);
			for ( ODFacet* facet = iter2.First(); iter2.IsNotComplete();
					facet = iter2.Next() )
			{
				ODHighlight highlight;
				
				if ( fSelected && activate )
					highlight = kODFullHighlight;
				else if ( fSelected && !activate )
					highlight = kODDimHighlight;
				else
					highlight = kODNoHighlight;
				
				facet->ChangeHighlight(ev, highlight);
			}
		}
	
	}
	
	this->DrawAll(ev, kODTrue);
}

//------------------------------------------------------------------------------
// Method:		GetSelectedRegion
//
// Description:	This method is called during
//------------------------------------------------------------------------------

RgnHandle CListItem::GetSelectedRegion(Environment* ev)
{
	RgnHandle selectionRgn = kODNULL;
	
	CListIterator iter(fFrameProxies);
	CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
	ODFrame* frame = proxy->GetFrame();
	
	if ( frame )
	{
		TempODShape embFrShape = frame->AcquireFrameShape(ev, kODNULL);
		selectionRgn = embFrShape->CopyQDRegion(ev);
		
		RgnHandle nameRgn = ODNewRgn();
		RectRgn(nameRgn, &fNameRect);
		OffsetRgn(nameRgn, -fItemOffset.h, -fItemOffset.v);
		
		UnionRgn(selectionRgn, nameRgn, selectionRgn);
		ODDisposeHandle((ODHandle)nameRgn);
	}
	
	return selectionRgn;
}

//------------------------------------------------------------------------------
// Method:		AddFrame
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::AddFrame(Environment* ev, ODFrame* displayFrame)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CEmbeddedFrameProxy* item = new CEmbeddedFrameProxy;
	item->InitEmbeddedFrameProxy(ev, displayFrame);
	
	fFrameProxies->Add(item);
}

//------------------------------------------------------------------------------
// Method:		RemoveFrame
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::RemoveFrame(Environment* ev, ODFrame* containingFrame)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CEmbeddedFrameProxy* proxy = kODNULL;
	
	CListIterator iter(fFrameProxies);
	for ( proxy=(CEmbeddedFrameProxy*)iter.First();iter.IsNotComplete();
			 proxy=(CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODFrame listFrame = frame->AcquireContainingFrame(ev);
		
		if ( ODObjectsAreEqual(ev, listFrame, containingFrame) )
			break;
	}
	
	if ( proxy != kODNULL )
	{
		fFrameProxies->Remove(proxy);
		proxy->Release(ev);
		delete proxy;
	}
}

//------------------------------------------------------------------------------
// Method:		FrameClosed
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::FrameClosed(Environment* ev, ODFrame* containingFrame)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODFrame listFrame = frame->AcquireContainingFrame(ev);
		
		if ( ODObjectsAreEqual(ev, containingFrame, listFrame) )
		{
			// Release the proxied frame.
			proxy->Release(ev);
			// Inflate the refcount to accomodate the implicit release
			// that occurs inside Close()
			ODAcquireObject(ev, frame);
			// Close the embedded frame.
			frame->Close(ev);
			// Remove and delete the proxy from our collection.
			iter.RemoveCurrent();
			delete proxy;
		}
	}
}

//------------------------------------------------------------------------------
// Method:		FrameRemoved
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::FrameRemoved(Environment* ev, ODFrame* containingFrame)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	ODFrame* frame = kODNULL;
	
	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODFrame listFrame = frame->AcquireContainingFrame(ev);
		
		if ( ODObjectsAreEqual(ev, containingFrame, listFrame) )
		{
			// Release the proxied frame.
			proxy->Release(ev);
			// Inflate the refcount to accomodate the implicit release
			// that occurs inside Remove()
			ODAcquireObject(ev, frame);
			// Remove the embedded frame.
			frame->Remove(ev);
			// Remove and delete the proxy from our collection.
			iter.RemoveCurrent();
			delete proxy;
		}
	}
}

//------------------------------------------------------------------------------
// Method:		AttachSourceFrame
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::AttachSourceFrame(	Environment*	ev,
									ODFrame*		containingFrame,
									ODFrame*		sourceContainingFrame )
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	ODFrame* listFrame = kODNULL;
	ODFrame* sourceListFrame = kODNULL;
	
	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODFrame listContainingFrame = frame->AcquireContainingFrame(ev);
		
		if ( !listFrame && listContainingFrame == containingFrame )
		{
			listFrame = frame;
		}
		if ( !sourceListFrame && listContainingFrame == sourceContainingFrame )
		{
			sourceListFrame = frame;
		}
	}
	
	if ( listFrame && sourceListFrame )
	{
		TempODPart part = listFrame->AcquirePart(ev);
		
		part->AttachSourceFrame(ev, listFrame, sourceListFrame);
	}
}

//------------------------------------------------------------------------------
// Method:		AddFacet
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::AddFacet(Environment* ev, ODFacet* containingFacet)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	ODFrame* frame = kODNULL;
	
	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy=(CEmbeddedFrameProxy*)iter.First();iter.IsNotComplete();
			 proxy=(CEmbeddedFrameProxy*)iter.Next() )
	{
		frame = proxy->GetFrame();
		TempODFrame containingFrame = frame->AcquireContainingFrame(ev);

		if ( ODObjectsAreEqual(ev, containingFrame, containingFacet->GetFrame(ev)) )
			break;
	}
	
	TempODShape clipShape =
					ODCopyAndRelease(ev, frame->AcquireFrameShape(ev, kODNULL));
	
	TempODTransform externalTransform = frame->CreateTransform(ev);
	externalTransform->SetQDOffset(ev, &fItemOffset);

	ODFacet* facet = containingFacet->CreateEmbeddedFacet(ev, frame, clipShape,
											externalTransform, kODNULL,
											kODNULL, kODNULL,
											kODFrameInFront);

	ODHighlight highlight;
	ODBoolean active = facet->GetWindow(ev)->IsActive(ev);

	if ( fSelected && active ) highlight = kODFullHighlight;
	else if ( fSelected && !active ) highlight = kODDimHighlight;
	else highlight = kODNoHighlight;
	
	facet->SetSelected(ev, fSelected);
	facet->ChangeHighlight(ev, highlight);
}

//------------------------------------------------------------------------------
// Method:		RemoveFacet
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::RemoveFacet(Environment* ev, ODFacet* containingFacet)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CList* tempList = new CList;
	ODFrame* frame = kODNULL;
	
	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		TempODFrameFacetIterator iter2(ev, frame);
		for ( ODFacet* facet = iter2.First(); iter2.IsNotComplete();
				facet = iter2.Next() )
		{
			if ( ODObjectsAreEqual(ev, facet->GetContainingFacet(ev), containingFacet) )
				tempList->Add((ODPtr)facet);
		}
	}
	
	CListIterator iter3(tempList);
	for ( ODFacet* facet = (ODFacet*)iter3.First();iter3.IsNotComplete();
			 facet = (ODFacet*)iter3.Next() )
	{
		containingFacet->RemoveFacet(ev, facet);
		iter3.RemoveCurrent();
		delete facet;
	}
	
	tempList->RemoveAllLinks();
	delete tempList;
}

//------------------------------------------------------------------------------
// Method:		Reattach
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::Reattach(Environment* ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		ODFrame* containingFrame = proxy->GetContainingFrame();
		
		// Reattach the frame to its containing frame.
		frame->SetContainingFrame(ev, containingFrame);
		
		// Add facets back the reattached frame.
		TempODFrameFacetIterator ffiter(ev, containingFrame);
		for ( ODFacet* facet = ffiter.First(); ffiter.IsNotComplete();
				facet = ffiter.Next() )
		{
			this->AddFacet(ev, facet);
		}
		
		// Forget (and release) the containing frame.
		proxy->SetContainingFrame(ev, kODNULL);
	}
}

//------------------------------------------------------------------------------
// Method:		Detach
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::Detach(Environment* ev)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	CListIterator iter(fFrameProxies);
	for ( CEmbeddedFrameProxy* proxy = (CEmbeddedFrameProxy*)iter.First();
			iter.IsNotComplete();
			proxy = (CEmbeddedFrameProxy*)iter.Next() )
	{
		ODFrame* frame = proxy->GetFrame();
		ODFrame* containingFrame = frame->AcquireContainingFrame(ev);
		
		// Remove all the facets to prevent drawing
		RemoveFacetsFromFrame(ev, frame); // (see PanelEditorUtils)
		
		// Cache the containing frame so we can reattach things later.
		proxy->SetContainingFrame(ev, containingFrame);
		
		// Set the containing frame to null to remove the frame from the
		// frame tree.
		frame->SetContainingFrame(ev, kODNULL);
		
		ODSafeReleaseObject(containingFrame);
	}
}

//------------------------------------------------------------------------------
// Method:		Read
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::Read(Environment* ev, ODStorageUnit* storageUnit)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	ODStorageUnitRef	strongRef;
	
	// Read item bounds
	StorageUnitGetValue(storageUnit, ev, sizeof(Point), (ODPtr)&fItemOffset);
	
	// Read List display frames
	TRY	
		StorageUnitGetValue(storageUnit, ev, kODStorageUnitRefSize, (ODPtr)&strongRef);
		
		if ( storageUnit->IsValidStorageUnitRef(ev, strongRef) )
		{		
			fPart = storageUnit->GetDraft(ev)->
					AcquirePart(ev, storageUnit->GetIDFromStorageUnitRef(ev, strongRef));
		}
	CATCH_ALL
		// exceptions consumed.
	ENDTRY

	// Get the name of the item we just read in.
	this->RetrieveName(ev);
	this->CalcNameRect();
}

//------------------------------------------------------------------------------
// Method:		Write
//
// Description:	This method is called during
//------------------------------------------------------------------------------

void CListItem::Write(Environment* ev, ODStorageUnit* storageUnit, ODDraft* draft, ODDraftKey key)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
	ASSERT(fFrameProxies != kODNULL, kAssertionFailed);

	ODStorageUnitRef	strongRef;
	ODStorageUnitID		partID;
	
	// Write list item bounds
	StorageUnitSetValue(storageUnit, ev, sizeof(Point), (ODPtr)&fItemOffset);

	TRY
		TempODPart part = this->AcquirePart(ev);
		partID = part->GetID(ev);
		
		// If a draft and key have been specified, the object is being cloned to
		// another draft. The cloned object should be strongly reference; not the
		// original.
		if ( draft != kODNULL )
			partID = draft->Clone(ev, key, partID, 0, 0);
		
		storageUnit->GetStrongStorageUnitRef(ev, partID, strongRef);
		StorageUnitSetValue(storageUnit, ev, kODStorageUnitRefSize, (ODPtr)&strongRef);
	CATCH_ALL
		// $$$$$ if we throw, what is the best thing to do? ask vincent.
	ENDTRY
}

//------------------------------------------------------------------------------
// Method:		ClonePart
//
// Description:	Public method called to clone the part to a interchange storage
//				unit.
//------------------------------------------------------------------------------

void CListItem::ClonePart(Environment* ev, ODStorageUnit* storageUnit, ODDraftKey key)
{
	ASSERT(fInited != kODFalse, kAssertionFailed);

	TempODPart part = this->AcquirePart(ev);
	part->CloneInto(ev, key, storageUnit, kODNULL);
}





