/*
	File:		Panel.h

	Contains:	Panel Classes Definition

	Written by:	Steve Smith
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
*/

// -- PanelEditor Includes --

#ifndef _SCROLLINGLIST_
#include "ScrollingList.h"
#endif

#ifndef _LISTITEM_
#include "ListItem.h"
#endif

#ifndef _SCROLLBAR_
#include "Scrollbar.h"
#endif

#ifndef _PANELEDITORDEF_
#include "PanelEditorDef.h"
#endif

#ifndef _PANELEDITORGLOBALS_
#include "PanelEditorGlobals.h"
#endif

#ifndef _PANELEDITORUTILS_
#include "PanelEditorUtils.h"
#endif

#ifndef _SAMPLECOLLECTIONS_
#include "SampleCollections.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#ifndef SOM_ODFacet_xh
#include <Facet.xh>
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

// -- OpenDoc Utilities --

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef _FOCUSLIB_
#include <FocusLib.h>
#endif

#ifndef _PLFMDEF_
#include <PlfmDef.h>
#endif

#ifndef _STORUTIL_
#include <StorUtil.h>
#endif

#ifndef _TEMPOBJ_
#include <TempObj.h>
#endif

#ifndef _TEMPITER_
#include <TempIter.h>
#endif


#pragma segment PanelEditorPanel

//==============================================================================
// CScrollingList
//==============================================================================

//------------------------------------------------------------------------------
// Method:		Constructor
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

CScrollingList::CScrollingList()
{
}

//------------------------------------------------------------------------------
// Method:		InitPanel
//
// Description:	This is the C++ class constructor.
//------------------------------------------------------------------------------

void CScrollingList::InitScrollingList(Environment* ev, PanelEditor* pe)
{
	fPE = pe;
	this->InitFrame();
}

//------------------------------------------------------------------------------
// Method:		Destructor
//
// Description:	This is the C++ class Destructor.
//------------------------------------------------------------------------------

CScrollingList::~CScrollingList()
{
	ASSERT(fInited != kODFalse, kAssertionFailed);
}

//------------------------------------------------------------------------------
// Method:		GetFrame
//
// Description:	This method is called in response to an update event.
//------------------------------------------------------------------------------

ODFrame* CScrollingList::GetFrame(Environment* ev, ODFrame* hintFrame)
{
	ODFrame* rtnFrame = kODNULL;

	CFrameListIterator fiter(fFrames);
	for ( ODFrame* frame=fiter.First();fiter.IsNotComplete();
			 frame=fiter.Next() )
	{
		// Calculate the panel frame's shape based on it's parent frame.
		TempODFrame parentFrame = frame->AcquireContainingFrame(ev);
		TempODFrame hintParent = hintFrame->AcquireContainingFrame(ev);
		
		if ( ODObjectsAreEqual(ev, frame, hintFrame) ||
				ODObjectsAreEqual(ev, hintParent, parentFrame) ||
				ODObjectsAreEqual(ev, hintFrame, parentFrame) )
		{
			rtnFrame = frame;
			break;
		}
	}

	return rtnFrame;
}

//------------------------------------------------------------------------------
// Method:		Draw
//
// Description:	This method is called in response to an update event.
//------------------------------------------------------------------------------

void CScrollingList::Draw(Environment* ev, ODFacet* facet)
{
	ODFrame* frame = facet->GetFrame(ev);
	
	// Frame the list.
	Rect box;
	GetQDFrameBounds(ev, frame, &box);
	
	PenNormal();
	FrameRect(&box);
	
	// Fill the list box (minus the scrollbar).
	InsetRect(&box,1,1);
	box.right -= 15;
	FillRect(&box,&ODQDGlobals.white);

	// draw scroll bar.
	CScrollbar* scrollbar = (CScrollbar*) facet->GetPartInfo(ev);
	scrollbar->Draw();
	
	// Draw panel names.
	{
		// Focus to the "content" so that the names draw
		// in the correctly scrolled positions.
		CFocusContent intiateDrawing(ev, facet);
		
		// Iterate over the embedded items and draw their names.
		COrdListIterator fiter(this->GetContent());
		for ( CListItem* item = (CListItem*)fiter.First();
				fiter.IsNotComplete();
				item = (CListItem*)fiter.Next() )
		{
			item->Draw(ev, facet);
		}
	}
	
}

//------------------------------------------------------------------------------
// Method:		GetFrameBounds
//
// Description:	This method is called in response to an update event.
//------------------------------------------------------------------------------

ODRect CScrollingList::GetFrameBounds(Environment* ev, ODFrame* containingFrame)
{
	ODSShort statusHeight = 0;
	ODRect bounds;
	TempODShape frameShape = containingFrame->AcquireFrameShape(ev, kODNULL);
	frameShape->GetBoundingBox(ev, &bounds);
	
	if ( containingFrame->IsRoot(ev) )
		statusHeight = 15;
	
	bounds.top = ff(0);
	bounds.left = ff(0);
	bounds.bottom = ff(FixedToInt(bounds.bottom-bounds.top)-2*kListTopEdge-statusHeight);
	bounds.right = ff(kListWidth);

	return bounds;
}

//------------------------------------------------------------------------------
// Method:		PrepareFrame
//
// Description:	This method is called in response to an update event.
//------------------------------------------------------------------------------

void CScrollingList::PrepareFrame( Environment* ev, ODFrame* frame )
{
	// Create frames for the embedded items.
	this->EmbedItemsInFrame(ev, frame);
	
	// If we are building the editor version of the part,
	// drag and drop is valid, so we need to set the frame's
	// drop flag.
#ifndef PE_ViewerBuild
	frame->SetDroppable(ev, kODTrue);
#endif
}

//------------------------------------------------------------------------------
// Method:		CleanupFrame
//
// Description:	This method is called in response to an update event.
//------------------------------------------------------------------------------

void CScrollingList::CleanupFrame( Environment* ev, ODFrame* frame,
									ODBoolean frameRemoved )
{
	// Close the embedded items.
	COrdListIterator fiter(this->GetContent());
	for ( CListItem* item = (CListItem*)fiter.First();
			fiter.IsNotComplete();
			item = (CListItem*)fiter.Next() )
	{
		if ( frameRemoved )
			item->FrameRemoved(ev, frame);
		else
			item->FrameClosed(ev, frame);
	}
}

//------------------------------------------------------------------------------
// Method:		CleanupFrame
//
// Description:	This method is called in response to an update event.
//------------------------------------------------------------------------------

void CScrollingList::SourceFrameAttached( Environment* ev, ODFrame* frame,
											ODFrame* sourceFrame)
{
	COrdListIterator fiter(this->GetContent());
	for ( CListItem* item = (CListItem*)fiter.First();
			fiter.IsNotComplete();
			item = (CListItem*)fiter.Next() )
	{
		item->AttachSourceFrame(ev, frame, sourceFrame);
	}
}

//------------------------------------------------------------------------------
// Method:		PrepareFacet
//
// Description:	This method is called in response to an update event.
//------------------------------------------------------------------------------

void CScrollingList::PrepareFacet(Environment* ev, ODFacet* facet)
{
	Rect ctrlRect;
	ODFrame* frame = facet->GetFrame(ev);
	
	GetQDFrameBounds(ev, frame, &ctrlRect);
	ctrlRect.left = ctrlRect.right - 16;

	// Create a new scrollbar object.
	CScrollbar* scrollbar = new CScrollbar;
	scrollbar->InitScrollbar(ev,facet,ctrlRect);
	// Dirty the scrollbar if there any embedded parts.
	scrollbar->Dirty(this->GetContent()->Count());

	// Store the scrollbar in the facet.
	facet->SetPartInfo(ev,(ODInfoType)scrollbar);
}

//------------------------------------------------------------------------------
// Method:		FacetChanged
//
// Description:	This method is called in response to an update event.
//------------------------------------------------------------------------------

void CScrollingList::FacetChanged(Environment* ev, ODFacet* facet)
{
	CScrollbar* scrollbar = (CScrollbar*) facet->GetPartInfo(ev);
	if ( scrollbar )
		scrollbar->Dirty(this->GetContent()->Count());
}

//------------------------------------------------------------------------------
// Method:		CleanupFacet
//
// Description:	This method is called in response to an update event.
//------------------------------------------------------------------------------

void CScrollingList::CleanupFacet(Environment* ev, ODFacet* facet)
{
	CScrollbar* scrollbar = (CScrollbar*) facet->GetPartInfo(ev);
	facet->SetPartInfo(ev, kODNULL);
	ODDeleteObject(scrollbar);
}

//------------------------------------------------------------------------------
// Method:		ClipItems
//
// Description:	This method is called...
//------------------------------------------------------------------------------

void CScrollingList::ClipItems(Environment* ev)
{
	CFrameListIterator fliter(fFrames);
	for ( ODFrame* frame=fliter.First();fliter.IsNotComplete();
			 frame=fliter.Next() )
	{
		TempODShape clipShape = ODCopyAndRelease(ev, frame->AcquireFrameShape(ev, kODNULL));
		
		ODCoordinate minusOne = ff(-1);
		clipShape->Outset(ev, minusOne);
	
		COrdListIterator oliter(this->GetContent());
		for ( CListItem* item = (CListItem*) oliter.First();
				oliter.IsNotComplete();
				item = (CListItem*)oliter.Next() )
		{
			Rect	itemRect;
			ODRect	bounds;
			
			ODUShort pos = item->GetListPosition();
			
			SetRect(&itemRect, kListLeftMargin, pos, kListLeftMargin+kODLargeIconSize,
						pos+kODLargeIconSize);
			IntToFixedRect(itemRect, bounds);
			
			TempODShape subtractShape = frame->CreateShape(ev);
			subtractShape->SetRectangle(ev, &bounds);
			clipShape->Subtract(ev, subtractShape);
		}
	}
}

//------------------------------------------------------------------------------
// Method:		AdjustScrollbar
//
// Description:	This method is called when
//------------------------------------------------------------------------------

void CScrollingList::AdjustItemPositions(Environment* ev)
{
	ODUShort pos = 0;
	
	COrdListIterator fiter(this->GetContent());
	for ( CListItem* item = (CListItem*)fiter.First();
			fiter.IsNotComplete();
			item = (CListItem*)fiter.Next() )
	{
		// Make sure the item knows its "current" list position.
		if ( item->GetListPosition() != pos )
			item->SetListPosition(ev, pos);
			
		pos++;
	}
	
	// Reclip the list items.
	this->ClipItems(ev);
}

//------------------------------------------------------------------------------
// Method:		AdjustScrollbar
//
// Description:	This method is called when
//------------------------------------------------------------------------------

void CScrollingList::AdjustScrollbar(Environment* ev)
{
	CFrameListIterator fiter(fFrames);
	for ( ODFrame* frame=fiter.First();fiter.IsNotComplete();
			 frame=fiter.Next() )
	{
		TempODFrameFacetIterator fiter(ev, frame);
		for ( ODFacet* facet = fiter.First(); fiter.IsNotComplete();
				facet = fiter.Next() )
		{
			// Retrieve the scrollbar.
			CScrollbar* scrollbar = (CScrollbar*) facet->GetPartInfo(ev);
			scrollbar->Adjust(this->GetContent()->Count());
		}
	}
}

//------------------------------------------------------------------------------
// Method:		HitTest
//
// Description:	This method is called when
//------------------------------------------------------------------------------

ODBoolean CScrollingList::HitTest(Environment* ev, ODFacet* facet, ODFacet* embFacet,
									Point pt, CListItem** item)
{
	// Select the embedded item.
	
	ODBoolean validHit = kODFalse;
	ODFrame* embFrame = kODNULL;
	
	// If the hit test specifies an embedded facet, extract the
	// frame from it and hit test with it too.
	if ( embFacet ) embFrame = embFacet->GetFrame(ev);

	// Translate the pt into content coordinate space by offsetting it
	// with the internal transform.
	TempODTransform intTransform =
						facet->GetFrame(ev)->AcquireInternalTransform(ev, kODNULL);
	Point offset = intTransform->GetQDOffset(ev);
	SubPt(offset,&pt);

	// Verify that the user's click is valid.
	COrdListIterator fiter(this->GetContent());
	for ( CListItem* listItem = (CListItem*)fiter.First();
			fiter.IsNotComplete();
			listItem = (CListItem*)fiter.Next() )
	{
		if ( listItem->HitTest(ev, embFrame, pt) &&
				!listItem->IsSelected() )
		{
			*item = listItem;
			validHit = kODTrue;
			break;
		}
	}
	
	return validHit;
}

//------------------------------------------------------------------------------
// Method:		EmbedItemsInFrame
//
// Description:	This method is called when
//------------------------------------------------------------------------------

CListItem* CScrollingList::EmbedPart(Environment* ev, ODPart* part)
{
	ODDraft* draft = ODGetDraft(ev,fPartDisplayed);

	CListItem* newItem = kODNULL; 
	ODVolatile(newItem);
	
	// Create a frame shape for the new item.
	ODRect	bounds;
	bounds.top = bounds.left = ff(0);
	bounds.bottom = ff(kODLargeIconSize);
	bounds.right = ff(kODLargeIconSize);

	// Create a proxy object for the new item.
	newItem = new CListItem;
	newItem->InitListItem();

	TRY
		CFrameListIterator fliter(fFrames);
		ODFrame* frame = fliter.First();
		
		TempODShape itemFrameShape = frame->CreateShape(ev);
		itemFrameShape->SetRectangle(ev, &bounds);
			
		// iterate over the list frames and embed the new item in each.
		for ( frame=fliter.First();fliter.IsNotComplete();
				 frame=fliter.Next() )
		{
			TempODFrame embeddedFrame =
					draft->CreateFrame(ev, kODFrameObject, frame, 
										itemFrameShape->Copy(ev),
										kODNULL, part, 
										gGlobals->fLargeIconView,
										gGlobals->fUndefinedPresentation,
										kODFalse, kODFalse);

			embeddedFrame->SetFrameGroup(ev, kListItemGroupID);
	
			// Add the frame to the list proxy.
			newItem->AddFrame(ev, embeddedFrame);
	
			// Iterate over the list frame's facets and add a facet to
			// the item for each.
			TempODFrameFacetIterator ffiter(ev, frame);
			for ( ODFacet* facet = ffiter.First(); ffiter.IsNotComplete();
					facet = ffiter.Next() )
			{
				newItem->AddFacet(ev, facet);
			}
		}
	
		// Set the defaults in the proxy.
		newItem->RetrieveName(ev);
		newItem->SetListPosition(ev, (ODUShort)(this->GetContent()->Count()));

	CATCH_ALL
		if ( newItem ) {	
			newItem->Release(ev);
			newItem = kODNULL;
		}
		RERAISE;
	ENDTRY
	
	return newItem;
}

//------------------------------------------------------------------------------
// Method:		EmbedItemsInFrame
//
// Description:	This method is called when
//------------------------------------------------------------------------------

void CScrollingList::EmbedItemsInFrame(Environment* ev, ODFrame* frame)
{
	ODDraft* draft = ODGetDraft(ev,fPartDisplayed);
	
	// Iterate over the embedded items and add new frames
	// for each inside the "parent" frame passed to this method.
	COrdListIterator oliter(this->GetContent());
	for ( CListItem* item = (CListItem*)oliter.First();
			oliter.IsNotComplete();
			item = (CListItem*)oliter.Next() )
	{
		ODRect bounds;
		bounds.top = bounds.left = ff(0);
		bounds.bottom = ff(kODLargeIconSize);
		bounds.right = ff(kODLargeIconSize);
	
		TempODShape itemFrameShape = frame->CreateShape(ev);
		itemFrameShape->SetRectangle(ev, &bounds);
		
		TempODPart embeddedPart = item->AcquirePart(ev);
	
		TempODFrame embeddedFrame =
				draft->CreateFrame(ev, kODNonPersistentFrameObject, frame, 
									itemFrameShape->Copy(ev),
									kODNULL, embeddedPart, 
									gGlobals->fLargeIconView,
									gGlobals->fUndefinedPresentation,
									kODFalse, kODFalse);

		embeddedFrame->SetFrameGroup(ev, kListItemGroupID);

		// Add the frame to the list proxy.
		item->AddFrame(ev, embeddedFrame);
	}
}








