/*
	File:		som_PanelEditor.cpp

	Contains:	Sample Part's SOM based part public interface implementation

	Written by:	Steve Smith
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
	
	----------------------------------------------------------------
	
	Notes:		Panel Editor is implemented using a "wrapped" class strategy.
				This means that the SOM class is just a smart delegator and
				the real implementation lives in a C++ class. The reason for
				doing this is twofold. First, this strategy all but elimintates
				the need for you to have to modify the IDL file and run the
				SOM compiler/emitter. Secondly, given that all code changes
				will occur in the C++ class, build turn around time is reduced
				to however fast the compiler can crunch code.
				
				This is not to say that implementing pure SOM classes isn't
				possible, nor necessary. There will be objects (ie. Extensions)
				which need to be exported for other parts to use which will need
				to be written in IDL and SOM implementation files.
*/

// Notification that this is a SOM source file
#define SampleCode_som_PanelEditor_Class_Source

// define underscore field names (ie. _fSelf)
#define VARIABLE_MACROS

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (ie. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with "ev" parameter set.
#include <Except.h>
#endif

// -- PanelEditor Includes --

#ifndef SOM_SampleCode_som_PanelEditor_xih
#include "som_PanelEditor.xih"
#endif

#ifndef _PANELEDITOR_
#include "PanelEditor.h"
#endif

#ifndef _PANELEDITORDEF_
#include "PanelEditorDef.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#ifndef _ODNEW_
#include <ODNew.h>
#endif

#pragma segment somPanelEditor

//====================================================================
// som_PanelEditor
//====================================================================

SOM_Scope void  SOMLINK som_PanelEditor__somInit(SampleCode_som_PanelEditor *somSelf)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__somInit");

	// somInit and somUninit methods behave like C++ constructors in that the
	// inherited methods are called automatically. Thus, there is no need to
	// call the parent class' somInit or somUninit.
	
	// There is also no need to set instance variables to zero/NULL
	// since SOM guarantees that a newly constructed object is zeroed.
	
	// Lastly, it is not valid to perform any operation here which may throw
	// an exception.
}

SOM_Scope void  SOMLINK som_PanelEditor__somUninit(SampleCode_som_PanelEditor *somSelf)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__somUninit");

	delete _fPart;
}

SOM_Scope void  SOMLINK som_PanelEditor__InitPart(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__InitPart");

	SOM_TRY
		// Create the real C++ SamplePart.		
		_fPart = new PanelEditor;

		SampleCode_som_PanelEditor_parent_ODPart_InitPart(somSelf,ev,storageUnit,partWrapper);
		_fPart->InitPart(ev,storageUnit,partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__InitPartFromStorage(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__InitPartFromStorage");

	SOM_TRY
		// Create the real C++ SamplePart.		
		_fPart = new PanelEditor;

		SampleCode_som_PanelEditor_parent_ODPart_InitPartFromStorage(somSelf,ev,storageUnit,partWrapper);
		_fPart->InitPartFromStorage(ev,storageUnit,partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__Externalize(SampleCode_som_PanelEditor *somSelf, Environment *ev)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__Externalize");

	SOM_TRY
		SampleCode_som_PanelEditor_parent_ODPart_Externalize(somSelf,ev);
		_fPart->Externalize(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__Release(SampleCode_som_PanelEditor *somSelf, Environment *ev)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__Release");

	SOM_TRY
		SampleCode_som_PanelEditor_parent_ODPart_Release(somSelf,ev);
		_fPart->Release(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__ReleaseAll(SampleCode_som_PanelEditor *somSelf, Environment *ev)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__ReleaseAll");

	SOM_TRY
		_fPart->ReleaseAll(ev);
		SampleCode_som_PanelEditor_parent_ODPart_ReleaseAll(somSelf,ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODSize  SOMLINK som_PanelEditor__Purge(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODSize size)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__Purge");

	ODSize bytes;

	SOM_TRY
		bytes = _fPart->Purge(ev,size); 
	SOM_CATCH_ALL
		bytes = 0;
	SOM_ENDTRY
	
	return bytes;
}

SOM_Scope void  SOMLINK som_PanelEditor__ReleaseExtension(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODExtension* extension)
{
    SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__ReleaseExtension");

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
	    SampleCode_som_PanelEditor_parent_ODPart_ReleaseExtension(somSelf,ev,extension);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODExtension*  SOMLINK som_PanelEditor__AcquireExtension(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODType extensionName)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__GetExtension");

	ODExtension* result;

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
		result = SampleCode_som_PanelEditor_parent_ODPart_AcquireExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODBoolean  SOMLINK som_PanelEditor__HasExtension(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODType extensionName)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__HasExtension");

	ODBoolean result;

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
		result = SampleCode_som_PanelEditor_parent_ODPart_HasExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_PanelEditor__FulfillPromise(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODStorageUnitView* /*promiseSUView*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__FulfillPromise");
}

SOM_Scope void  SOMLINK som_PanelEditor__DropCompleted(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODPart* ,
		ODDropResult dropResult)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__DropCompleted");
}

SOM_Scope ODDragResult  SOMLINK som_PanelEditor__DragEnter(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODDragItemIterator* dragInfo,
		ODFacet* facet,
		ODPoint* where)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__DragEnter");

	ODDragResult result;

	SOM_TRY
		result = _fPart->DragEnter(ev,dragInfo,facet,where);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODDragResult  SOMLINK som_PanelEditor__DragWithin(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODDragItemIterator* dragInfo,
		ODFacet* facet,
		ODPoint* where)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__DragWithin");

	ODDragResult result;
	
	SOM_TRY
		result = _fPart->DragWithin(ev,dragInfo,facet,where);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_PanelEditor__DragLeave(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFacet* facet,
		ODPoint* where)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__DragLeave");

	SOM_TRY
		_fPart->DragLeave(ev,facet,where);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODDropResult  SOMLINK som_PanelEditor__Drop(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODDragItemIterator* dropInfo,
		ODFacet* facet,
		ODPoint* where)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__Drop");

	ODDropResult result;

	SOM_TRY
		result = _fPart->Drop(ev,dropInfo,facet,where);
	SOM_CATCH_ALL
		result = kODDropFail;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODBoolean  SOMLINK som_PanelEditor__RevealFrame(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODShape* /*revealShape*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__RevealFrame");

	return kODFalse;
}

SOM_Scope ODBoolean  SOMLINK som_PanelEditor__EditInLinkAttempted(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
    SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__EditInLinkAttempted");

    return kODFalse;
}

SOM_Scope void  SOMLINK som_PanelEditor__EmbeddedFrameUpdated(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODUpdateID change)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__EmbeddedFrameChanged");
}

SOM_Scope void  SOMLINK som_PanelEditor__EmbeddedFrameSpec(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODObjectSpec* /*spec*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__EmbeddedFrameSpec");
}

SOM_Scope void  SOMLINK som_PanelEditor__ContainingPartPropertiesUpdated(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnit* propertyUnit)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__ContainingPartPropertiesChanged");
}

SOM_Scope ODEmbeddedFramesIterator*  SOMLINK som_PanelEditor__CreateEmbeddedFramesIterator(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__CreateEmbeddedFramesIterator");

	// This is temporary fix for scripting. This method will be implemented soon.
	SampleCode_som_PanelEditor_parent_ODPart_CreateEmbeddedFramesIterator(somSelf,ev,frame);
	return kODNULL;
}

SOM_Scope void  SOMLINK som_PanelEditor__DisplayFrameAdded(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__DisplayFrameAdded");

	SOM_TRY
		_fPart->DisplayFrameAdded(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__DisplayFrameRemoved(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__DisplayFrameRemoved");

	SOM_TRY
		_fPart->DisplayFrameRemoved(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__DisplayFrameConnected(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__DisplayFrameConnected");

	SOM_TRY
		_fPart->DisplayFrameConnected(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__DisplayFrameClosed(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__DisplayFrameClosed");

	SOM_TRY
		_fPart->DisplayFrameClosed(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__AttachSourceFrame(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODFrame* sourceFrame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__AttachSourceFrame");

	SOM_TRY
		_fPart->AttachSourceFrame(ev,frame,sourceFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__FrameShapeChanged(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__FrameShapeChanged");

	SOM_TRY
		_fPart->FrameShapeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__ViewTypeChanged(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__ViewTypeChanged");

	SOM_TRY
		_fPart->ViewTypeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__PresentationChanged(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__PresentationChanged");
}

SOM_Scope void  SOMLINK som_PanelEditor__SequenceChanged(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__SequenceChanged");
}

SOM_Scope void  SOMLINK som_PanelEditor__ClonePartInfo(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODDraftKey key,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView,
		ODFrame* scopeFrame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__ClonePartInfo");

	SOM_TRY
		_fPart->ClonePartInfo(ev,key,partInfo,storageUnitView,scopeFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__WritePartInfo(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__WritePartInfo");

	SOM_TRY
		_fPart->WritePartInfo(ev,partInfo,storageUnitView);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODInfoType  SOMLINK som_PanelEditor__ReadPartInfo(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnitView* storageUnitView)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__ReadPartInfo");

	ODInfoType result;

	SOM_TRY
		result = _fPart->ReadPartInfo(ev,frame,storageUnitView);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODID  SOMLINK som_PanelEditor__Open(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__Open");

	ODID result;

	SOM_TRY
		result = _fPart->Open(ev,frame);
	SOM_CATCH_ALL
		result = 0;
	SOM_ENDTRY

	return result;
}


SOM_Scope ODFrame*  SOMLINK som_PanelEditor__RequestEmbeddedFrame(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* containingFrame,
		ODFrame* baseFrame,
		ODShape* frameShape,
		ODPart* embeddedPart,
		ODTypeToken viewType,
		ODTypeToken presentation,
		ODBoolean isOverlaid)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__RequestEmbeddedFrame");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_PanelEditor__RemoveEmbeddedFrame(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__RemoveEmbeddedFrame");
}

SOM_Scope ODShape*  SOMLINK som_PanelEditor__RequestFrameShape(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* embeddedFrame,
		ODShape* frameShape)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__RequestFrameShape");

	ODShape* result;
	
	SOM_TRY
		result = _fPart->RequestFrameShape(ev,embeddedFrame,frameShape);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_PanelEditor__UsedShapeChanged(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__UsedShapeChanged");
}

SOM_Scope ODStorageUnit*  SOMLINK som_PanelEditor__AcquireContainingPartProperties(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__GetContainingPartProperties");

	return kODNULL;
}

SOM_Scope ODShape*  SOMLINK som_PanelEditor__AdjustBorderShape(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFacet* embeddedFacet,
		ODShape* shape)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__AdjustBorderShape");

	ODShape* result;
	
	SOM_TRY
		result = _fPart->AdjustBorderShape(ev,embeddedFacet,shape);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_PanelEditor__FacetAdded(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFacet* facet)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__FacetAdded");

	SOM_TRY
		_fPart->FacetAdded(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__FacetRemoved(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFacet* facet)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__FacetRemoved");

	SOM_TRY
		_fPart->FacetRemoved(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__CanvasChanged(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFacet* /*facet*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__CanvasChanged");
}

SOM_Scope void  SOMLINK som_PanelEditor__GeometryChanged(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFacet* facet,
		ODBoolean clipShapeChanged,
		ODBoolean externalTransformChanged)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__GeometryChanged");

	SOM_TRY
		_fPart->GeometryChanged(ev,facet,clipShapeChanged,externalTransformChanged);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__Draw(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFacet* facet,
		ODShape* invalidShape)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__Draw");

	SOM_TRY
		_fPart->Draw(ev,facet,invalidShape);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__CanvasUpdated(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODCanvas* /*canvas*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__CanvasUpdated");
}

SOM_Scope void  SOMLINK som_PanelEditor__HighlightChanged(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFacet* facet)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__HighlightChanged");

	SOM_TRY
		_fPart->HighlightChanged(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODULong  SOMLINK som_PanelEditor__GetPrintResolution(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__GetPrintResolution");

	return kMinImagingResolution;
}

SOM_Scope ODLinkSource*  SOMLINK som_PanelEditor__CreateLink(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODByteArray* /*data*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__CreateLink");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_PanelEditor__LinkUpdated(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODLink* link,
		ODUpdateID change)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__LinkUpdated");
}

SOM_Scope void  SOMLINK som_PanelEditor__RevealLink(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODLinkSource* /*linkSource*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__RevealLink");
}

SOM_Scope void  SOMLINK som_PanelEditor__LinkStatusChanged(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__LinkStatusChanged");
}

SOM_Scope ODBoolean  SOMLINK som_PanelEditor__BeginRelinquishFocus(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__BeginRelinquishFocus");

	ODBoolean result;

	SOM_TRY
		result = _fPart->BeginRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_PanelEditor__CommitRelinquishFocus(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__CommitRelinquishFocus");

	SOM_TRY
		_fPart->CommitRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__AbortRelinquishFocus(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__AbortRelinquishFocus");

	SOM_TRY
		_fPart->AbortRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__FocusAcquired(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__FocusAcquired");

	SOM_TRY
		_fPart->FocusAcquired(ev,focus,ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__FocusLost(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__FocusLost");

	SOM_TRY
		_fPart->FocusLost(ev,focus,ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__CloneInto(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODDraftKey key,
		ODStorageUnit* toSU,
		ODFrame* scope)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__CloneInto");

	SOM_TRY
		SampleCode_som_PanelEditor_parent_ODPart_CloneInto(somSelf,ev,key,toSU,scope);
		_fPart->CloneInto(ev,key,toSU,scope);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__ExternalizeKinds(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODTypeList* /*kindset*/)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__ExternalizeKinds");
}

SOM_Scope void  SOMLINK som_PanelEditor__ChangeKind(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODType kind)
{
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__ChangeKind");
}

SOM_Scope ODBoolean  SOMLINK som_PanelEditor__HandleEvent(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODEventData* event,
		ODFrame* frame,
		ODFacet* facet,
		ODEventInfo* eventInfo)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__HandleEvent");

	ODBoolean result;

	SOM_TRY
		result = _fPart->HandleEvent(ev,event,frame,facet,eventInfo);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_PanelEditor__AdjustMenus(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__AdjustMenus");

	SOM_TRY
		_fPart->AdjustMenus(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__UndoAction(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODActionData* actionState)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__UndoAction");

	SOM_TRY
		_fPart->UndoAction(ev,actionState);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__RedoAction(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODActionData* actionState)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__RedoAction");

	SOM_TRY
		_fPart->RedoAction(ev,actionState);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_PanelEditor__DisposeActionState(SampleCode_som_PanelEditor *somSelf, Environment *ev,
		ODActionData* actionState,
		ODDoneState doneState)
{
	SampleCode_som_PanelEditorData *somThis = SampleCode_som_PanelEditorGetData(somSelf);
	SampleCode_som_PanelEditorMethodDebug("SampleCode_som_PanelEditor","som_PanelEditor__DisposeActionState");

	SOM_TRY
		_fPart->DisposeActionState(ev,actionState,doneState);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

