/*
	File:		som_SoundEditor.cpp

	Contains:	SoundEditor's SOM part public interface implementation

	Written by:	Steve Smith and Andrey Dolgachev
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.

	----------------------------------------------------------------
	
	Notes:		SoundEditor is implemented using a "wrapped" class strategy.
				This means that the SOM class is just a smart delegator and
				the real implementation lives in a C++ class. The reason for
				doing this is twofold. First, this strategy all but elimintates
				the need for you to have to modify the IDL file and run the
				SOM compiler/emitter. Secondly, given that all code changes
				will occur in the C++ class, build turn around time is reduced
				to however fast the compiler can crunch code.
				
				This is not to say that implementing pure SOM classes isn't
				possible, nor necessary. There will be objects (ie. Extensions)
				which need to be exported for other parts to use which will need
				to be written in IDL and SOM implementation files.
*/

// Notification that this is a SOM source file
#define SampleCode_som_SoundEditor_Class_Source

// define underscore field names
#define VARIABLE_MACROS

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (eg. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with the "ev" parameter set.
#include <Except.h>
#endif

// -- SoundEditor Includes --

#ifndef SOM_SampleCode_som_SoundEditor_xih
#include "som_SoundEditor.xih"
#endif

#ifndef _SOUNDEDITOR_
#include "SoundEditor.h"
#endif

#ifndef _SOUNDEDITORDEF_
#include "SoundEditorDef.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

// -- OpenDoc Includes --

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#pragma segment somSoundEditor

//====================================================================
// som_SoundEditor
//====================================================================

SOM_Scope void  SOMLINK som_SoundEditor__somInit(SampleCode_som_SoundEditor *somSelf)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__somInit");

	// somInit and somUninit methods behave like C++ constructors in that the
	// inherited methods are called automatically.
	// Thus, there is no need to call the parent class' somInit or somUninit.
	
	// There is also no need to set instance variables to zero/NULL
	// since SOM guarantees that a newly constructed object is zeroed.
	
	// Lastly, it is not valid to perform any operation here which may throw
	// an exception.
}

SOM_Scope void  SOMLINK som_SoundEditor__somUninit(SampleCode_som_SoundEditor *somSelf)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__somUninit");

	ODDeleteObject(_fPart);
}

SOM_Scope ODSize  SOMLINK som_SoundEditor__Purge(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODSize size)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__Purge");

	ODSize result;

	SOM_TRY
		result = _fPart->Purge(ev,size);
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_SoundEditor__InitPart(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__InitPart");

	SOM_TRY
		// Create the real C++ SoundEditor.		
		_fPart = new SoundEditor;

		SampleCode_som_SoundEditor_parent_ODPart_InitPart(somSelf,ev,storageUnit,partWrapper);
		_fPart->InitPart(ev,storageUnit,partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__InitPartFromStorage(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__InitPartFromStorage");

	SOM_TRY
		// Create the real C++ SoundEditor.		
		_fPart = new SoundEditor;

		SampleCode_som_SoundEditor_parent_ODPart_InitPartFromStorage(somSelf, ev, storageUnit, partWrapper);
		_fPart->InitPartFromStorage(ev,storageUnit,partWrapper);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__Externalize(SampleCode_som_SoundEditor *somSelf, Environment *ev)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__Externalize");
	
	SOM_TRY
		SampleCode_som_SoundEditor_parent_ODPart_Externalize(somSelf, ev);
		_fPart->Externalize(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__Release(SampleCode_som_SoundEditor *somSelf, Environment *ev)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__Release");
	
	SOM_TRY
		SampleCode_som_SoundEditor_parent_ODPart_Release(somSelf, ev);
		_fPart->Release(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__ReleaseAll(SampleCode_som_SoundEditor *somSelf, Environment *ev)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__ReleaseAll");

	SOM_TRY
		_fPart->ReleaseAll(ev);
		SampleCode_som_SoundEditor_parent_ODPart_ReleaseAll(somSelf,ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODBoolean  SOMLINK som_SoundEditor__HasExtension(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODType extensionName)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__HasExtension");

	ODBoolean result = kODFalse;
	
	SOM_TRY
		result = _fPart->HasExtension(ev, extensionName);

		// If we dont support this extension, see if one of our parent classes might.
		if (result == kODFalse)
			result = SampleCode_som_SoundEditor_parent_ODPart_HasExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return result;
}

SOM_Scope ODExtension*  SOMLINK som_SoundEditor__AcquireExtension(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODType extensionName)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__AcquireExtension");

	ODExtension* result = kODNULL;

	SOM_TRY
		result = _fPart->AcquireExtension(ev, extensionName);

		// If we dont support this extension, see if one of our parent classes might.
		if (result == kODNULL)
			result = SampleCode_som_SoundEditor_parent_ODPart_AcquireExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_SoundEditor__ReleaseExtension(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODExtension* extension)
{
    SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
    SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__ReleaseExtension");

	SOM_TRY
		if (!_fPart->ReleaseExtension(ev, extension))
			SampleCode_som_SoundEditor_parent_ODPart_ReleaseExtension(somSelf,ev,extension);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__FulfillPromise(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODStorageUnitView* promiseSUView)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__FulfillPromise");
}

SOM_Scope void  SOMLINK som_SoundEditor__DropCompleted(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODPart* ,
		ODDropResult dropResult)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__DropCompleted");
}

SOM_Scope ODDragResult  SOMLINK som_SoundEditor__DragEnter(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODDragItemIterator* /*dragInfo*/,
		ODFacet* /*facet*/,
		ODPoint* /*where*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__DragEnter");

	return kODFalse;
}

SOM_Scope ODDragResult  SOMLINK som_SoundEditor__DragWithin(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODDragItemIterator* /*dragInfo*/,
		ODFacet* /*facet*/,
		ODPoint* /*where*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__DragWithin");

	return kODFalse;
}

SOM_Scope void  SOMLINK som_SoundEditor__DragLeave(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFacet* /*facet*/,
		ODPoint* /*where*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__DragLeave");
}

SOM_Scope ODDropResult  SOMLINK som_SoundEditor__Drop(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODDragItemIterator* /*dropInfo*/,
		ODFacet* /*facet*/,
		ODPoint* /*where*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__Drop");

	return kODDropFail;
}

SOM_Scope void  SOMLINK som_SoundEditor__ContainingPartPropertiesUpdated(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/,
		ODStorageUnit* /*propertyUnit*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__ContainingPartPropertiesUpdated");
}

SOM_Scope void  SOMLINK som_SoundEditor__DisplayFrameAdded(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__DisplayFrameAdded");

	SOM_TRY
		_fPart->DisplayFrameAdded(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__DisplayFrameRemoved(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__DisplayFrameRemoved");

	SOM_TRY
		_fPart->DisplayFrameRemoved(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__DisplayFrameConnected(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__DisplayFrameConnected");

	SOM_TRY
		_fPart->DisplayFrameConnected(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__DisplayFrameClosed(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__DisplayFrameClosed");

	SOM_TRY
		_fPart->DisplayFrameClosed(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__AttachSourceFrame(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODFrame* sourceFrame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__AttachSourceFrame");

	SOM_TRY
		_fPart->AttachSourceFrame(ev,frame,sourceFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__FrameShapeChanged(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__FrameShapeChanged");

	SOM_TRY
		_fPart->FrameShapeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__ViewTypeChanged(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__ViewTypeChanged");

	SOM_TRY
		_fPart->ViewTypeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__PresentationChanged(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__PresentationChanged");
}

SOM_Scope void  SOMLINK som_SoundEditor__SequenceChanged(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__SequenceChanged");
}

SOM_Scope void  SOMLINK som_SoundEditor__ClonePartInfo(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODDraftKey key,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView,
		ODFrame* scopeFrame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__ClonePartInfo");

	SOM_TRY
		_fPart->ClonePartInfo(ev,key,partInfo,storageUnitView,scopeFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__WritePartInfo(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__WritePartInfo");

	SOM_TRY
		_fPart->WritePartInfo(ev,partInfo,storageUnitView);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODInfoType  SOMLINK som_SoundEditor__ReadPartInfo(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnitView* storageUnitView)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__ReadPartInfo");

	ODInfoType result;

	SOM_TRY
		result = _fPart->ReadPartInfo(ev,frame,storageUnitView);
	SOM_CATCH_ALL
	SOM_ENDTRY

	return result;
}

SOM_Scope ODID  SOMLINK som_SoundEditor__Open(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__Open");

	ODID result = kODNULLID;

	SOM_TRY
		result = _fPart->Open(ev,frame);
	SOM_CATCH_ALL
		result = kODNULLID;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_SoundEditor__FacetAdded(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFacet* facet)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__FacetAdded");

	SOM_TRY
		_fPart->FacetAdded(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__FacetRemoved(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFacet* facet)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__FacetRemoved");

	SOM_TRY
		_fPart->FacetRemoved(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__CanvasChanged(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFacet* /*facet*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__CanvasChanged");
}

SOM_Scope void  SOMLINK som_SoundEditor__GeometryChanged(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFacet* facet,
		ODBoolean clipShapeChanged,
		ODBoolean externalTransformChanged)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__GeometryChanged");

	SOM_TRY
		_fPart->GeometryChanged(ev,facet,clipShapeChanged,externalTransformChanged);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__Draw(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFacet* facet,
		ODShape* invalidShape)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__Draw");

	SOM_TRY
		_fPart->Draw(ev, facet, invalidShape);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__CanvasUpdated(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODCanvas* /*canvas*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__CanvasUpdated");
}

SOM_Scope void  SOMLINK som_SoundEditor__HighlightChanged(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFacet* facet)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__HighlightChanged");

	SOM_TRY
		_fPart->HighlightChanged(ev, facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODULong  SOMLINK som_SoundEditor__GetPrintResolution(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__GetPrintResolution");

	return kMinImagingResolution;
}

SOM_Scope ODLinkSource*  SOMLINK som_SoundEditor__CreateLink(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODByteArray* /*data*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__CreateLink");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_SoundEditor__LinkUpdated(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODLink* ,
		ODUpdateID change)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__LinkUpdated");
}

SOM_Scope void  SOMLINK som_SoundEditor__RevealLink(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODLinkSource* /*linkSource*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__RevealLink");
}

SOM_Scope void  SOMLINK som_SoundEditor__LinkStatusChanged(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__LinkStatusChanged");
}

SOM_Scope ODBoolean  SOMLINK som_SoundEditor__BeginRelinquishFocus(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__BeginRelinquishFocus");

	ODBoolean result;

	SOM_TRY
		result = _fPart->BeginRelinquishFocus(ev, focus, ownerFrame, proposedFrame);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_SoundEditor__CommitRelinquishFocus(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__CommitRelinquishFocus");

	SOM_TRY
		_fPart->CommitRelinquishFocus(ev, focus, ownerFrame, proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__AbortRelinquishFocus(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__AbortRelinquishFocus");

	SOM_TRY
		_fPart->AbortRelinquishFocus(ev, focus, ownerFrame, proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__FocusAcquired(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__FocusAcquired");

	SOM_TRY
		_fPart->FocusAcquired(ev, focus, ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__FocusLost(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__FocusLost");

	SOM_TRY
		_fPart->FocusLost(ev, focus, ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__CloneInto(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODDraftKey key,
		ODStorageUnit* toSU,
		ODFrame* scope)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__CloneInto");

	SOM_TRY
		SampleCode_som_SoundEditor_parent_ODPart_CloneInto(somSelf,ev, key, toSU, scope);
		_fPart->CloneInto(ev, key, toSU, scope);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__ExternalizeKinds(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODTypeList* kindset)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__ExternalizeKinds");

	SOM_TRY
		_fPart->ExternalizeKinds(ev, kindset);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__ChangeKind(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODType kind)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__ChangeKind");

	SOM_TRY
		_fPart->ChangeKind(ev, kind);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODBoolean  SOMLINK som_SoundEditor__HandleEvent(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODEventData* event,
		ODFrame* frame,
		ODFacet* facet,
		ODEventInfo* eventInfo)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__HandleEvent");

	ODBoolean result;
	
	SOM_TRY
		result = _fPart->HandleEvent(ev, event, frame, facet, eventInfo);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_SoundEditor__AdjustMenus(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODFrame* frame)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__AdjustMenus");

	SOM_TRY
		_fPart->AdjustMenus(ev, frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_SoundEditor__UndoAction(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODActionData* /*actionState*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__UndoAction");
}

SOM_Scope void  SOMLINK som_SoundEditor__RedoAction(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODActionData* /*actionState*/)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__RedoAction");
}

SOM_Scope void  SOMLINK som_SoundEditor__DisposeActionState(SampleCode_som_SoundEditor *somSelf, Environment *ev,
		ODActionData* /*actionState*/,
		ODDoneState doneState)
{
//	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__DisposeActionState");
}

SOM_Scope SoundEditor*  SOMLINK som_SoundEditor__GetImplementation(SampleCode_som_SoundEditor *somSelf, Environment *ev)
{
	SampleCode_som_SoundEditorData *somThis = SampleCode_som_SoundEditorGetData(somSelf);
	SampleCode_som_SoundEditorMethodDebug("SampleCode_som_SoundEditor","som_SoundEditor__GetImplementation");
	
	return _fPart;
}
